# Data analysis library numpy and pandas
import pandas as pd
import numpy as np

# Data visualization library matplotlib and seaborn
import seaborn as sns
import matplotlib
import matplotlib.pyplot as plt
import matplotlib.font_manager
import matplotlib.patches as mpatches

import mimetypes
import urllib
import os

#from google.colab import drive	
#drive.mount('/content/drive')

## setting path
# get current path
path_current = os.getcwd()
# the path is where the dataset saved

path_current=path_current.replace('\\', '/')
path = path_current + '/Data/Box/' 
# the "path_img" is the position where final image will be saved
path_img = path_current + '/Images/'

class Box_plot: 
  # read data
  def read_file(self,file):
    file_url = urllib.request.pathname2url(file)
    ftype = mimetypes.guess_type(file_url, strict=True)[0]
    ## read data file according to its formate, default includes three types of files: csv/excel/text
    # read csv format data from the parking dataset
    if 'excel' in ftype:
      # usecols: return a subset of the columns, here choose one column to use in the line chart
      data = pd.read_csv(path+file)
    # read excel format data from the parking dataset
    elif 'sheet' in ftype:
      data = pd.read_excel(path+file)
    # read text format data from the parking dataset
    elif ftype == 'text/plain':
      data = pd.read_csv(path+file, sep="\t")
    else:
      print("File type cannot find!")
    return data

  # check the available file name
  # if the input file name already existed then rename to file_1, file_2 
  def get_available_name(self,filename):
    n=[1]
    def check_meta(file_name):
        file_name_new=file_name
        if file_name in [os.path.splitext(i)[0] for i in os.listdir(path_img)]:   
            file_name_new=file_name+'_'+str(n[0])
            n[0]+=1
        if file_name_new in [os.path.splitext(i)[0] for i in os.listdir(path_img)]:   
            file_name_new=check_meta(file_name)
        return file_name_new
    available_name=check_meta(filename)
    return available_name

  # file: file name of your data source
  # x_col_name: ['x_column_name_a','x_column_name_b'...]
  # paper_type : 'single' or 'double'
  def box(self, file, x_col_name, paper_type, **kwargs):
    # Configuration of the box plot
    # plotwidth: width of the plot
    # plotheight: height of the plot
    # backgrid: backgrid of the plot
    # isframe: frame of the plot
    # my_font: the typeface of x, y labels
    # gridlinewidth: grid line width
    # title: True or False as options. If it is True, add title for the plot
    # title_pad: if the title is True, modify pad size of title
    # title_size: if the title is True, modify size of title
    # title_loc: if the title is True, modify location of title
    # x_label: the content of x label
    # y_label: the content of y label
    # palette: color palette of box plot
    # widths: each box width
    # notch: True or false, if true, add notch to each box
    # vert: True or False, If True , makes the boxes vertical. If False, everything is drawn horizontally
    # showfliers: True or False, if true, show the outliers beyond the caps
    # showcaps: True or False, if true, show the caps on the ends of whiskers
    # patch_artist: True or False, if true, boxes and drawn with Patch artists
    # box_linestyle: the style of box outline
    # box_linewidth: the line width of box outlines
    # medianline_color: the color or median line color
    # showmeans: True or False, if true, show the median line
    # meanline_color: the color or mean line color
    # meanline_style: the line style of mean line
    # markers_shape: the shape of outliers
    # actual_value: True or False, if true show the actual value of mean or meadian
    # save_image: True or False as options. If it is True, save chart
    # savefig_bbox_inches: Bounding box in inches
    # file_name: the file name in saving image

    single_column_conf={ 'plotwidth':8,#weight
                      'plotheight':6, #height
                      'my_font':'DejaVu Sans',
                      'labeltext_size':25,
                      'labelpad':10,
                      'backgrid':True,
                      'isframe':True,
                      'gridlinewidth':0.5,
                      'title':False,
                      'title_pad':10,
                      'title_size':20,
                      'tick_size':15,
                      'title_loc':'center',
                      'x_label':None,
                      'y_label':None,
                      'palette':sns.color_palette("hls", 10),
                      'widths':0.5,
                      'notch':False,
                      'vert':True,
                      'showfliers':True,
                      'showcaps':True,
                      'patch_artist':True,
                      'box_linestyle': '-', 
                      'box_linewidth': 1,
                      'medianline_color':'Black',
                      'showmeans':False,
                      'meanline_color':'White',
                      'meanline_style':'--',
                      'markers_shape':['o','v','D','X','P','2','p','x','d','4','<','*'], 
                      'actual_value':False,
                      'save_image':False,
                      'savefig_bbox_inches':'tight',
                      'file_name':'box_plot',
                      }
    double_column_conf={ 'plotwidth':8,#weight
                      'plotheight':6, #height
                      'my_font':'DejaVu Sans',
                      'labeltext_size':25,
                      'labelpad':10,
                      'backgrid':True,
                      'isframe':True,
                      'gridlinewidth':0.5,
                      'title':False,
                      'title_pad':10,
                      'title_size':20,
                      'tick_size':18,
                      'title_loc':'center',
                      'x_label':None,
                      'y_label':None,
                      'palette':sns.color_palette("hls", 10),
                      'widths':0.5,
                      'notch':False,
                      'vert':True,
                      'showfliers':True,
                      'showcaps':True,
                      'patch_artist':True,
                      'box_linestyle': '-', 
                      'box_linewidth': 1,
                      'medianline_color':'Black',
                      'showmeans':False,
                      'meanline_color':'White',
                      'meanline_style':'--',
                      'markers_shape':['o','v','D','X','P','2','p','x','d','4','<','*'], 
                      'actual_value':False,
                      'save_image':False,
                      'savefig_bbox_inches':'tight',
                      'file_name':'box_plot',
                      }
    # choose the configuration of plot
    if paper_type == 'single':
      conf = single_column_conf
    elif paper_type == 'double':
      conf = double_column_conf  
    conf.update(kwargs)
    # create figure and set figure size  
    fig, ax_left = plt.subplots(figsize = (conf['plotwidth'], conf['plotheight']))   
    
    #read data
    try:
      data = self.read_file(file)
    except Exception:
      print('Sorry, this file does not exist, please check the file name') 
    # draw back grid of whole plot
    if conf['backgrid'] == True:
      ax_left.grid(linestyle="--", linewidth=conf['gridlinewidth'], color='gray', alpha=0.5)
    
    # x, y axis label setting
    # fontsize: x, y title size
    ax_left.set_xlabel(conf['x_label'], fontproperties=conf['my_font'], fontsize=conf['labeltext_size'], labelpad=conf['labelpad'])
    ax_left.set_ylabel(conf['y_label'], fontproperties=conf['my_font'], fontsize=conf['labeltext_size'], labelpad=conf['labelpad'])

    #plot     
    for i in range(len(x_col_name)):
      # if the showmeans is false, draw the box plot only includes details of medians
      if conf['showmeans'] == False:
        box=ax_left.boxplot(data[x_col_name[i]],positions=[i],labels=[x_col_name[i]],patch_artist=conf['patch_artist'],vert=conf['vert'],widths=conf['widths'],
                        boxprops={'facecolor':conf['palette'][i],'color':conf['palette'][i],'linestyle': conf['box_linestyle'], 'linewidth': conf['box_linewidth']},
                        medianprops={'color':conf['medianline_color']},
                        flierprops=dict(marker=conf['markers_shape'][i],markeredgecolor=conf['palette'][i]),
                        notch=conf['notch']
                        )
        # show the actual value of medians in box plot
        if conf['actual_value']==True:
          ax_left.text(x=box['medians'][0].get_data(orig=True)[0][1]+0.01,y=box['medians'][0].get_data(orig=True)[1][0],s='medians='+str(box['medians'][0].get_data(orig=True)[1][0]),fontsize=12)

        
      else:
        box=ax_left.boxplot(data[x_col_name[i]],positions=[i],labels=[x_col_name[i]],patch_artist=conf['patch_artist'],vert=conf['vert'],widths=conf['widths'],
                      boxprops={'facecolor':conf['palette'][i],'color':conf['palette'][i],'linestyle': conf['box_linestyle'], 'linewidth': conf['box_linewidth']},
                      medianprops={'color':conf['medianline_color'],'linewidth': conf['box_linewidth']},
                      showmeans=True,meanline=True,meanprops={'color':conf['meanline_color'],'linestyle': conf['meanline_style'],'linewidth': conf['box_linewidth']},
                      flierprops=dict(marker=conf['markers_shape'][i],markeredgecolor=conf['palette'][i]), 
                      notch=conf['notch']
                      )
        if conf['actual_value']==True:
          # check the value of medians and means the actual values shows above the line if it has bigger value, else, show detail below the line
          if box['medians'][0].get_data(orig=True)[1][0]>box['means'][0].get_data(orig=True)[1][0]:
            ax_left.text(x=box['medians'][0].get_data(orig=True)[0][0],y=box['medians'][0].get_data(orig=True)[1][0]*1.1,s='medians='+str(round(box['medians'][0].get_data(orig=True)[1][0],3)),fontsize=12)
            ax_left.text(x=box['means'][0].get_data(orig=True)[0][0],y=box['means'][0].get_data(orig=True)[1][0]*0.8,s='means='+str(round(box['means'][0].get_data(orig=True)[1][0],3)),fontsize=12)
          else:
            ax_left.text(x=box['medians'][0].get_data(orig=True)[0][0],y=box['medians'][0].get_data(orig=True)[1][0]*0.8,s='medians='+str(round(box['medians'][0].get_data(orig=True)[1][0],3)),fontsize=12)
            ax_left.text(x=box['means'][0].get_data(orig=True)[0][0],y=box['means'][0].get_data(orig=True)[1][0]*1.1,s='means='+str(round(box['means'][0].get_data(orig=True)[1][0],3)),fontsize=12)
    
   
    # if False, top and right borders removing    
    if conf['isframe'] == False:
        ax.spines['top'].set_visible(False)
        ax.spines['right'].set_visible(False)
    # set x, y tick's direction, default:out, can be set to in,out,inout
    for tick in ax_left.xaxis.get_major_ticks():
      tick.label.set_fontsize(conf['tick_size'])
    for tick in ax_left.yaxis.get_major_ticks():
      tick.label.set_fontsize(conf['tick_size'])
    #title setting
    if conf['title'] == False:
      pass
    else:
      ax_left.set_title(conf['title'], fontsize=conf['title_size'], loc=conf['title_loc'], pad=conf['title_pad'])
    #save
    if conf['save_image'] == True:
      file_name=conf['file_name']
      file_newname = self.get_available_name(file_name)
      plt.savefig(path_img+file_newname, bbox_inches=conf['savefig_bbox_inches'],dpi=600,format='jpg')  
    plt.show()


## import python data analysis library
import pandas as pd

## import data visualization library matplotlib and seaborn
import seaborn as sns
import matplotlib
import matplotlib.pyplot as plt
from matplotlib.ticker import MultipleLocator, FormatStrFormatter, AutoMinorLocator, FuncFormatter
from matplotlib.ticker import AutoMinorLocator, MultipleLocator, FuncFormatter
from matplotlib import rc
from mpl_toolkits.axes_grid.inset_locator import (inset_axes, InsetPosition,
                                                  mark_inset,
                                                  zoomed_inset_axes)
import matplotlib.cm as cm
import numpy as np
import matplotlib.patches as mpatches
from sklearn.linear_model import LinearRegression

## import module to read files
import mimetypes
import urllib
import os

path_current = os.getcwd()

path_current=path_current.replace('\\', '/')
# the path is where the dataset saved
path = path_current + '/Data/Sca' 
# the "path_img" is the position where final image will be saved
path_img = path_current + '/Images/'
class Scatter:
    # read data
    def read_file(self, file):
        """Read different types of files and return pandas dataframe.

        This function can transform multiple file type such as csv/excel/text into
        a pandas dataframe. User need to input the filename such as: 'file.csv'.
        If the file type cannot be find or not support it, it will return the message.

        Args:
            file: filename of user data source

        Returns:
            data : A pandas dataframe

        """
        # Get the file URL
        file_url = urllib.request.pathname2url(file)  # try catch
        # use mimetypes package to guess the file type
        # For example: 'file.csv' will return 'excel'
        ftype = mimetypes.guess_type(file_url, strict=True)[0]
        ## read data file according to its formate, default includes three types of files: csv/excel/text
        # read csv format data
        if 'excel' in ftype:
            # usecols: return a subset of the columns, here choose one column to use in the line chart
            data = pd.read_csv(path + file)
        # read excel format data
        elif 'sheet' in ftype:
            data = pd.read_excel(path + file)
        # read text format data
        elif ftype == 'text/plain':
            data = pd.read_csv(path + file, sep="\t")
        else:
            print("File type cannot find!")
        return data

    def linear_trend(self, data, x_col_name, y_col_name):
        """Add the trend line for the scatter plot

        The trend line is caculated it by sklearn.linear_model. When the user input trend_line=True in scatter_plot
        method, it draw the trend line based on that scatter plot.

        Args:
            data: The pandas dataframe
            x_col_name : The X column name
            y_col_name : The Y column name

        """

        # reshape the X and Y value to fit linear regressor
        X = data[x_col_name].values.reshape(-1, 1)
        Y = data[y_col_name].values.reshape(-1, 1)

        # using linear_regressor to calcaulate the Trend line
        linear_regressor = LinearRegression()
        linear_regressor.fit(X, Y)
        Y_pred = linear_regressor.predict(X)
        plt.plot(X, Y_pred, color='#20639B')

    def scatter_plot(self,
                     file,
                     x_col_name,
                     y_col_name,
                     x_label,
                     y_label,
                     output_name=None,
                     **kwargs):
        """Basic scatter plot with trend line or not

        This function will convert the user's incoming data set into DataFrame and then draw
        the scatter Chart using Matplotlib. This scatter chart can show x, y and trend line.
        The basic scatter plot is used to plot the relationship bewteen two variables. The user can
        select trend_line=True to use the trend line.

        Args:
            file: file name of your data source
            x_col_name: The X column name
            y_col_name: The Y column name
            x_label: x axis label
            y_label: y axis label
            output_name : the output file name. The deafult output name is dataset name. The output file type is PDF. For example, The output file name would be :'dataset1.pdf'

        """
        conf = {
            'plotwidth':
            6,  # weight
            'plotheight':
            4.5,  # height
            'my_font':
            'DejaVu Sans',
            'backgrid':
            True,
            'isframe':
            True,
            'trend_line':
            False,
            'linewidth':
            2,
            'gridlinewidth':
            0.5,
            'gridline_style':
            '--',
            'gridline_color':
            'gray',
            'gridline_alpha':
            0.5,
            'labeltext_size':
            20,
            'labelpad':
            10,
            'legend_size':
            10,
            'scatter_size':
            100,
            'legend_loc':
            'upper right',
            'title':
            False,
            'title_pad':
            10,
            'title_size':
            15,
            'title_loc':
            'center',
            'xtrick_fontsize':
            15,
            'ytrick_fontsize':
            15,
            'bubble_marker':
            'o',
            'alpha':
            0.8,
            'colors_list': [
                "#3d405b", "#e07a5f", "#f4f1de", "#81b29a", "#f2cc8f",
                "#264653", "#2a9d8f", "#e9c46a", "#f4a261", "#e76f51",
                '#ECE133', '#56B4E9'
            ],
            'plot_title':''
        }

        # when new configuraton is set, update the original one
        conf.update(kwargs)
        ## create figure and set figure size
        fig, ax = plt.subplots(figsize=(conf['plotwidth'], conf['plotheight']))
        ## background grid setting
        if conf['backgrid'] == True:
            ax.grid(linestyle=conf['gridline_style'],
                    linewidth=conf['gridlinewidth'],
                    color=conf['gridline_color'],
                    alpha=conf['gridline_alpha'])

        ## x, y axis setting
        # fontsize: x, y title size
        ax.set_xlabel(x_label,
                      fontproperties=conf['my_font'],
                      fontsize=conf['labeltext_size'],
                      labelpad=conf['labelpad'])
        ax.set_ylabel(y_label,
                      fontproperties=conf['my_font'],
                      fontsize=conf['labeltext_size'],
                      labelpad=conf['labelpad'])

        # Input the valid filename and return the pandas dataframe for drawing
        data = self.read_file(file)

        # scatter plot
        ax.scatter(data[x_col_name],
                   data[y_col_name],
                   s=conf['scatter_size'],
                   c=conf['colors_list'][0])

        # add the trend line
        # The default is not with trend line, The user need to type trend_line=True in the parameter
        if conf['trend_line'] == True:
            self.linear_trend(data, x_col_name, y_col_name)

        # xtricks and yticks font size
        plt.xticks(fontsize=conf['xtrick_fontsize'])
        plt.yticks(fontsize=conf['ytrick_fontsize'])

        # title
        plt.title(conf['plot_title'], fontsize=conf['title_size'])

        # if the output name is empty, the output name will be file name
        if output_name == None:
            output_name = file.split(".")[0]

        # save the plot
        plt.tight_layout()
        plt.savefig(path_img + output_name)

        # show the plot
        plt.show()

    def scatter_category(self,
                         file,
                         x_col_name,
                         y_col_name,
                         x_label,
                         y_label,
                         category_col,
                         output_name=None,
                         **kwargs):
        """scatter plot with differnt category

        This function will convert the user's incoming data set into DataFrame and then draw
        the scatter Chart using Matplotlib. This scatter chart can show x, y and different color for different category scatter.

        The user have to input three vaiables column inculding x_col_name, y_col_name and category_col. The category_col
        will make each scatter being different color and also have the legend labels.


        Agrs:
            file: file name of your data source
            x_col_name: The X column name
            y_col_name: The Y column name
            category_col : The category of each scatter
            x_label: x axis label
            y_label: y axis label
            output_name : the output file name. The deafult output name is dataset name. The output file type is PDF. For example, The output file name would be :'dataset1.pdf'


        """

        conf = {
            'plotwidth':
            6,  # weight
            'plotheight':
            4.5,  # height
            'my_font':
            'DejaVu Sans',
            'backgrid':
            True,
            'isframe':
            True,
            'trend_line':
            False,
            'linewidth':
            2,
            'gridlinewidth':
            0.5,
            'gridline_style':
            '--',
            'gridline_color':
            'gray',
            'gridline_alpha':
            0.5,
            'labeltext_size':
            20,
            'labelpad':
            10,
            'legend_size':
            10,
            'scatter_size':
            100,
            'legend_loc':
            'upper right',
            'title':
            False,
            'title_pad':
            10,
            'title_size':
            15,
            'title_loc':
            'center',
            'xtrick_fontsize':
            15,
            'ytrick_fontsize':
            15,
            'bubble_marker':
            'o',
            'alpha':
            0.8,
            'colors_list': [
                "#3d405b", "#e07a5f", "#f4f1de", "#81b29a", "#f2cc8f",
                "#264653", "#2a9d8f", "#e9c46a", "#f4a261", "#e76f51",
                '#ECE133', '#56B4E9'
            ],
            'plot_title':'',
            'ylim_setting':True,
            'ylim_buttom':0
        }

        # when new configuraton is set, update the original one
        conf.update(kwargs)
        ## create figure and set figure size
        fig, ax = plt.subplots(figsize=(conf['plotwidth'], conf['plotheight']))
        ## background grid setting
        if conf['backgrid'] == True:
            ax.grid(linestyle=conf['gridline_style'],
                    linewidth=conf['gridlinewidth'],
                    color=conf['gridline_color'],
                    alpha=conf['gridline_alpha'])

        ## x, y axis setting
        # fontsize: x, y title size
        ax.set_xlabel(x_label,
                      fontproperties=conf['my_font'],
                      fontsize=conf['labeltext_size'],
                      labelpad=conf['labelpad'])
        ax.set_ylabel(y_label,
                      fontproperties=conf['my_font'],
                      fontsize=conf['labeltext_size'],
                      labelpad=conf['labelpad'])

        # Input the valid filename and return the pandas dataframe for drawing
        data = self.read_file(file)

        map_category = []
        # create the list to get the category unique value
        # Example: ['Asia' 'Europe' 'Africa' 'Americas' 'Oceania']
        category_list = data[category_col].unique()

        # map each category item to one color
        # {'Asia': 'r', 'Europe': 'b', 'Africa': 'y','Americas':'g','Oceania':'c'}
        map_category = dict(
            zip(category_list, conf['colors_list'][:len(category_list)]))

        # color for each item
        category = data[category_col].map(map_category)

        # plot the scatter plot
        scatter = ax.scatter(data[x_col_name],
                             data[y_col_name],
                             s=conf['scatter_size'],
                             color=category,
                             label=category,
                             alpha=conf['alpha'],
                             edgecolors='grey')

        # create the mptaches list to for legend
        match_category = []
        for key, value in map_category.items():
            match_category.append(mpatches.Patch(color=value, label=key))
        # crate the legend
        ax.legend(handles=match_category,fontsize=conf['legend_size'])

        # xtricks and yticks font size
        plt.xticks(fontsize=conf['xtrick_fontsize'])
        plt.yticks(fontsize=conf['ytrick_fontsize'])

        # set ylim buttom
        if conf['ylim_setting'] == True:
          ax.set_ylim(bottom=conf['ylim_buttom'])

        # title
        plt.title(conf['plot_title'], fontsize=conf['title_size'])

        # if the output name is empty, the output name will be file name
        if output_name == None:
            output_name = file.split(".")[0]

        # save the plot
        plt.tight_layout()
        plt.savefig(path_img + output_name)

        # show the plot
        plt.show()

        # file: file name of your data source

    # x_col_name: The X column name
    # y_col_name: The Y column name
    # category_col : The category of each scatter
    # x_label: x axis label
    # y_label: y axis label
    # plot_title : The title of the plot
    # output_name : the output file name. The deafult output name is dataset name. The output file type is PDF. For example, The output file name would be :'dataset1.pdf'

    def scatter_marker(self,
                       file,
                       x_col_name,
                       y_col_name,
                       x_label,
                       y_label,
                       category_col,
                       output_name=None,
                       **kwargs):
        """scatter plot with differnt marker

        This function will convert the user's incoming data set into DataFrame and then draw
        the scatter Chart using Matplotlib. This scatter chart can show x, y and different marker for different category scatter.

        The user have to input three vaiables column inculding x_col_name, y_col_name and category_col. The category_col
        will make each scatter being different marker and also have the legend labels.


        Agrs:
            file: file name of your data source
            x_col_name: The X column name
            y_col_name: The Y column name
            category_col : The category of each scatter
            x_label: x axis label
            y_label: y axis label
            output_name : the output file name. The deafult output name is dataset name. The output file type is PDF. For example, The output file name would be :'dataset1.pdf'


        """

        conf = {
            'plotwidth':
            6,  # weight
            'plotheight':
            4.5,  # height
            'my_font':
            'DejaVu Sans',
            'backgrid':
            True,
            'isframe':
            True,
            'trend_line':
            False,
            'linewidth':
            2,
            'gridlinewidth':
            0.5,
            'gridline_style':
            '--',
            'gridline_color':
            'gray',
            'gridline_alpha':
            0.5,
            'labeltext_size':
            20,
            'labelpad':
            10,
            'legend_size':
            10,
            'scatter_size':
            100,
            'legend_loc':
            'upper right',
            'title':
            False,
            'title_pad':
            10,
            'title_size':
            15,
            'title_loc':
            'center',
            'xtrick_fontsize':
            15,
            'ytrick_fontsize':
            15,
            'bubble_marker':
            'o',
            'alpha':
            0.8,
            'colors_list': [
                "#3d405b", "#e07a5f", "#f4f1de", "#81b29a", "#f2cc8f",
                "#264653", "#2a9d8f", "#e9c46a", "#f4a261", "#e76f51",
                '#ECE133', '#56B4E9'
            ],
            'marker_type': ['o', 'x', '+', 'v', '^', 's', '*'],
            'plot_title':''
        }

        # when new configuraton is set, update the original one
        conf.update(kwargs)
        ## create figure and set figure size
        fig, ax = plt.subplots(figsize=(conf['plotwidth'], conf['plotheight']))
        ## background grid setting
        if conf['backgrid'] == True:
            ax.grid(linestyle=conf['gridline_style'],
                    linewidth=conf['gridlinewidth'],
                    color=conf['gridline_color'],
                    alpha=conf['gridline_alpha'])

        ## x, y axis setting
        # fontsize: x, y title size
        ax.set_xlabel(x_label,
                      fontproperties=conf['my_font'],
                      fontsize=conf['labeltext_size'],
                      labelpad=conf['labelpad'])
        ax.set_ylabel(y_label,
                      fontproperties=conf['my_font'],
                      fontsize=conf['labeltext_size'],
                      labelpad=conf['labelpad'])

        # Input the valid filename and return the pandas dataframe for drawing
        data = self.read_file(file)

        map_category = []
        # create the list to get the category unique value
        # Example: ['Asia' 'Europe' 'Africa' 'Americas' 'Oceania']
        category_list = data[category_col].unique()

        # map each category item to one color
        # {'Asia': 'r', 'Europe': 'b', 'Africa': 'y','Americas':'g','Oceania':'c'}
        map_category = dict(
            zip(category_list, conf['marker_type'][:len(category_list)]))

        # separete differnt color for each item
        category = data[category_col].map(map_category)

        # scatter plot with multiple marker
        # This for loop is going to draw different markers based on categories
        # For example, it will plot category 1 for marker 'x' and then plot category 2 for marker 'o'.
        for kind in map_category:
            data_category = data[data[category_col] == kind]
            plt.scatter(data_category[x_col_name],
                        data_category[y_col_name],
                        s=conf['scatter_size'],
                        marker=map_category[kind],
                        alpha=conf['alpha'],
                        edgecolors='grey')

        # add category(different marker) legend
        unique_list = data[category_col].unique()
        plt.legend(labels=unique_list,fontsize=conf['legend_size'])

        # xtricks and yticks font size
        plt.xticks(fontsize=conf['xtrick_fontsize'])
        plt.yticks(fontsize=conf['ytrick_fontsize'])

        # title
        plt.title(conf['plot_title'], fontsize=conf['title_size'])

        # if user did not input the output file name, it will use input file name as output file name
        if output_name == None:
            output_name = file.split(".")[0]

        # save the plot
        plt.tight_layout()
        plt.savefig(path_img + output_name)



import pandas as pd
import numpy as np

# Data visualization library matplotlib and seaborn
import seaborn as sns
import matplotlib
import matplotlib.pyplot as plt
import matplotlib.font_manager

import mimetypes
import urllib
import os


#from google.colab import drive	
#drive.mount('/content/drive')

## setting path
# get current path
path_current = os.getcwd()
# the path is where the dataset saved

path_current=path_current.replace('\\', '/')
path = path_current + '/Data/Pyr/' 
# the "path_img" is the position where final image will be saved
path_img = path_current + '/Images/'

class pyramid:
    # read data
  def read_file(self,file):
    file_url = urllib.request.pathname2url(file)
    ftype = mimetypes.guess_type(file_url, strict=True)[0]
    ## read data file according to its formate, default includes three types of files: csv/excel/text
    # read csv format data from the parking dataset
    if 'excel' in ftype:
      # usecols: return a subset of the columns, here choose one column to use in the line chart
      data = pd.read_csv(path+file)
    # read excel format data from the parking dataset
    elif 'sheet' in ftype:
      data = pd.read_excel(path+file)
    # read text format data from the parking dataset
    elif ftype == 'text/plain':
      data = pd.read_csv(path+file, sep="\t")
    else:
      print("File type cannot find!")
    return data

  # check the available file name
  # if the input file name already existed then rename to file_1, file_2
  def get_available_name(self,filename):
    n=[1]
    def check_meta(file_name):
      file_name_new=file_name
      if file_name in [os.path.splitext(i)[0] for i in os.listdir(path_img)]:   
          file_name_new=file_name+'_'+str(n[0])
          n[0]+=1
      if file_name_new in [os.path.splitext(i)[0] for i in os.listdir(path_img)]:   
          file_name_new=check_meta(file_name)
      return file_name_new
    available_name=check_meta(filename)
    return available_name

  # file: file name of your data source
  # x_col_name: the columns name of x axis
  # y_col_name: the columns name of y axis  
  # group_col: the category columns
  # paper_type : 'single' or 'double'
  def pyramid(self,file,x_col_name,y_col_name,group_col,paper_type ,**kwargs):  
    # Configuration of the pyramid chart
    # plotwidth: width of the plot
    # plotheight: height of the plot
    # my_font: the typeface of x, y labels
    # labeltext_size: text size of x,y labels
    # labelpad: pad size of label
    # title: True or False as options. If it is True, add title for the plot
    # title_pad: if the title is True, modify pad size of title
    # title_size: if the title is True, modify size of title
    # title_loc: if the title is True, modify location of title
    # color: the colors of pyramid
    # axis_scale: get more blank in x,y axis
    # sort: True or False, if true, sort the value in ascending order
    # x_label: the content of x label
    # y_label: the content of y label
    # xlabel_rotate: the rotation of x axis labels
    # ylabel_rotate: the rotation of y axis labels
    # tick_size: set the size of ticks
    # legend_loc: location of legend
    # legend_textsize: size of legend text
    # annotate_textsize: the font size of annotation
    # x_pos_parameter: get the relative x position of annotation,
    # y_pos_parameter: get the relative y position of annotation,
    # save_image: True or False as options. If it is True, save chart
    # savefig_bbox_inches: Bounding box in inches
    # file_name: the file name in saving image

    single_column_conf={ 'plotwidth':8,
                      'plotheight':6, 
                      'my_font':'DejaVu Sans',
                      'labeltext_size':16,
                      'labelpad':10,
                      'title':False,
                      'title_pad':10,
                      'title_size':20,
                      'title_loc':'center',
                      'color':['skyblue','khaki'],
                      'axis_scale':2,
                      'sort':False,
                      'x_label':None,
                      'y_label':None,
                      'xlabel_rotate':0,
                      'ylabel_rotate':0,
                      'tick_size':13,
                      'annotate':False,
                      'legend_loc':'upper right',
                      'legend_textsize':14,
                      'annotate_textsize':10,
                      'x_pos_parameter':2.5,
                      'y_pos_parameter':10,
                      'save_image':False,
                      'savefig_bbox_inches':'tight',
                      'file_name':'pyramid'
                      }

    double_column_conf={ 'plotwidth':8,
                      'plotheight':6, 
                      'my_font':'DejaVu Sans',
                      'labeltext_size':17,
                      'labelpad':10,
                      'title':False,
                      'title_pad':10,
                      'title_size':20,
                      'title_loc':'center',
                      'color':['skyblue','khaki'],
                      'axis_scale':2,
                      'sort':False,
                      'x_label':None,
                      'y_label':None,
                      'xlabel_rotate':0,
                      'ylabel_rotate':0,
                      'tick_size':14,
                      'annotate':False,
                      'legend_loc':'upper right',
                      'legend_textsize':15,
                      'annotate_textsize':10,
                      'x_pos_parameter':2.5,
                      'y_pos_parameter':10,
                      'save_image':False,
                      'savefig_bbox_inches':'tight',
                      'file_name':'pyramid'
                      }                  

    # choose configuration
    if paper_type == 'single':
      conf = single_column_conf
    elif paper_type == 'double':
      conf = double_column_conf  

  # when new configuraton is set, update the original one
    conf.update(kwargs)  
    ## create figure and set figure size  
    fig, ax_left = plt.subplots(figsize = (conf['plotwidth'], conf['plotheight']))
 
    # read file 
    try:
      data = self.read_file(file)
    except Exception:
      print('Sorry, this file does not exist, please check the file name')   
    
    #plot
    #check if there are repetitive records in y columns
    data=data.groupby([group_col[0], y_col_name[0]], as_index=False).sum()
    
    # if two category values have same plus or minus characteristic, change one of them
    if len(data[group_col[0]].unique()) ==2:
      cate=data[group_col[0]].unique()
      
      if (data.loc[data[group_col[0]] == cate[0],x_col_name] > 0).all().values != (data.loc[data[group_col[0]] == cate[1],x_col_name] > 0).all().values:
        pass
      else:
        data.loc[data[group_col[0]]==cate[0], x_col_name]=data.loc[data[group_col[0]]==cate[0], x_col_name].apply(lambda x: -x)

    # give each category one color   
    color=conf['color']
    group_col=group_col[0]
    # create pyramid by barplot
    # if sort is false draw directly
    if conf['sort']==False:      
      for c, group in zip(color, data[group_col].unique()):       
        g = sns.barplot(x=x_col_name[0],y=y_col_name[0],data=data.loc[data[group_col]==group, :],color=c,label=group)
    # sort each line by comparing their total length  
    else:
      data['abs']=data[x_col_name[0]].abs()
      df=data[[y_col_name[0],'abs']].groupby(y_col_name[0],as_index=False).sum()
      df.sort_values('abs',inplace=True)
      sorted=list(df[y_col_name[0]])
      data.index=data[y_col_name[0]]
      data=data.loc[sorted]
      # draw pyramid by using sorted data
      for c, group in zip(color, data[group_col].unique()):        
        g = sns.barplot(x=x_col_name[0],y=y_col_name[0],data=data.loc[data[group_col]==group, :],color=c,label=group)
    # get more blank in x, y axis   
    ax_left_xlim = ax_left.get_xlim()         
    ax_left.set_xlim(ax_left_xlim[0]*conf['axis_scale'],ax_left_xlim[1]*conf['axis_scale'])
    ax_left_ylim = ax_left.get_ylim()         
    ax_left.set_ylim(ax_left_ylim[0],ax_left_ylim[1]*conf['axis_scale'])
    # set all value in x axis to be positive
    ax_left.set_xticklabels([str(abs(int(x))) for x in ax_left.get_xticks()])

    # annotate actual value
    if conf['annotate']==True:  
      for p in g.patches:  
        # annotate the area in the left of chart
        if p.get_width()<0:          
          g.annotate(format(abs(p.get_width()), '.1f'), 
                    (len(str(p.get_width())) + p.get_width() , p.get_y()+p.get_height()), 
                    ha = 'center', va = 'center', 
                    xytext = (-len(str(p.get_width()))*conf['x_pos_parameter'],p.get_height()*conf['y_pos_parameter']), 
                    textcoords = 'offset points',fontsize=conf['annotate_textsize'])
        # annotate the area in the right of chart
        else:          
          g.annotate(format(abs(p.get_width()),'.1f'), 
                    ( p.get_width() , p.get_y()+p.get_height()), 
                    ha = 'center', va = 'center', 
                    xytext = (len(str(p.get_width()))*conf['x_pos_parameter'],  p.get_height()*conf['y_pos_parameter']), 
                    textcoords = 'offset points',fontsize=conf['annotate_textsize'])

    # x,y label setting
    ax_left.set_xlabel(conf['x_label'], fontproperties=conf['my_font'], fontsize=conf['labeltext_size'], labelpad=conf['labelpad'])
    ax_left.set_ylabel(conf['y_label'], fontproperties=conf['my_font'], fontsize=conf['labeltext_size'], labelpad=conf['labelpad'])  
    # x,y ticks setting
    for tick in ax_left.xaxis.get_major_ticks():
      tick.label.set_fontsize(conf['tick_size'])
    for tick in ax_left.yaxis.get_major_ticks():
      tick.label.set_fontsize(conf['tick_size'])
    # rotate x,y axis
    if conf['xlabel_rotate'] !=0:
      plt.setp(ax_left.get_xticklabels(), rotation=conf['xlabel_rotate'], ha="right",
            rotation_mode="anchor")
    if conf['ylabel_rotate'] !=0:  
      plt.setp(ax_left.get_yticklabels(), rotation=conf['ylabel_rotate'], ha="right",
            rotation_mode="anchor")
    # legend setting
    plt.legend( fontsize=conf['legend_textsize'],loc=conf['legend_loc'])
    # title setting
    if conf['title'] == False:
      pass
    else:
      ax_left.set_title(conf['title'], fontsize=conf['title_size'], loc=conf['title_loc'], pad=conf['title_pad'])
    #save plot setting
    if conf['save_image'] == True:
      file_name=conf['file_name']
      file_newname = self.get_available_name(file_name)
      plt.savefig(path_img+file_newname, bbox_inches=conf['savefig_bbox_inches'],dpi=600,format='jpg') 
     
    # showing the image
    plt.show()


#pip install geopandas
#pip install geoplot
# import the geospatial data visualization package
import os
from pip._internal import main

main.main(['install', 'geopandas'])
main.main(['install', 'geoplot'])
import geoplot as gplt
import geoplot.crs as gcrs
import matplotlib.pyplot as plt
from matplotlib.colors import LinearSegmentedColormap
import pandas as pd

import urllib
import mimetypes
import mapclassify as mc
#from google.colab import drive
import folium	
from IPython.display import display
import warnings
warnings.filterwarnings('ignore')
#drive.mount('/content/drive')

## setting path
# get current path
path_current = os.getcwd()

path_current=path_current.replace('\\', '/')
# the path is where the dataset saved
path = path_current + '/Data/Map/' 
# the "path_img" is the position where final image will be saved
path_img = path_current + '/Images/'

class Map:
    # read data
    def read_file(self, file, longtitude, latitude):
        """Read different types of files and return geopandas dataframe.

        This function can transform multiple file type such as csv/geojson into
        a pandas dataframe. User need to input the filename such as: 'file.csv'.
        If the file type cannot be find or not support it, it will return the message.

        Args:
            file: filename of user data source

        Returns:
            data : A geopandas dataframe

        """
        file_url = urllib.request.pathname2url(file)  # try catch
        ftype = file_url.split(".")[-1]
        ## read data file according to its formate, default includes three types of files: csv/geojson
        # read csv format data from the parking dataset
        if 'excel' in ftype:
            # read the csv file into the pandas dataframe
            csv = pd.read_csv(path + file)
            # transfer the pandas dataframe become geopandas dataframe base on longtitude and latitude
            data = gpd.GeoDataFrame(csv,
                                    geometry=gpd.points_from_xy(
                                        csv[longtitude], csv[latitude]))
        # transfer geojson data into geopandas dataframe
        elif 'geojson' in ftype:
            data = gpd.read_file(path + file)
        # warning for the incorect file
        else:
            print("File type cannot find!")
        return data

    def read_routefile(self, file):
        """Read different types of files and return pandas dataframe.

        This function can transform multiple file type such as csv/excel/text into
        a pandas dataframe. User need to input the filename such as: 'file.csv'.
        If the file type cannot be find or not support it, it will return the message.

        Args:
            file: filename of user data source

        Returns:
            data : A pandas dataframe

        """
        # Get the file URL
        file_url = urllib.request.pathname2url(file)  # try catch
        # use mimetypes package to guess the file type
        # For example: 'file.csv' will return 'excel'
        ftype = mimetypes.guess_type(file_url, strict=True)[0]
        ## read data file according to its formate, default includes three types of files: csv/excel/text
        # read csv format data
        if 'excel' in ftype:
            data = pd.read_csv(path + file)
        # read excel format data
        elif 'sheet' in ftype:
            data = pd.read_excel(path + file)
        # read text format data from
        elif ftype == 'text/plain':
            data = pd.read_csv(path + file, sep="\t")
        else:
            print("File type cannot find!")
        return data

    # file: file name of your data source
    # x_col_name: ['index'] or ['x_column_name_a','x_column_name_b'...]
    # y_col_name: ['y_column_name_a','y_column_name_b'...]
    # x_label: x axis label
    # y_label: y axis label
    # legend_label: legend labels names
    # plot_type: type of plot: stack or simple
    # paper_type : 'single' or 'double'
    def map_plot(self, base_file, output_name, **kwargs):
        """Geospatial data visualization map with multiple plot

        To use this function, it have to import the base_file which contain the polygon shape.
        The user can select multiple overlay type such as point, kde, sankey,choropleth,path.
        The inital overlay type is point(scatter).

        Args:
            base_file: The file name of base file, it need to conatin ploygons.
            output_name: The output file name

        """

        conf = {
            'plotwidth': 10,  # weight,
            'file_longtitude': 'longtitude',
            'file_latitude': 'latitude',
            'polyplot_linewidth': 1,
            'polyploy_facecolor': 'lightgrey',
            'polyplot_figsize': (16, 12),
            'kde_cmap': 'Reds',
            'kde_shade': True,
            'kde_shade_lowest': True,
            'kde_alpha': 0.8,
            'pointplot_edgecolor': 'black',
            'pointplot_linewidth': 0.5,
            'pointplot_alpha': 0.8,
            'pointplot_legend': True,
            'choropleth_figsize': (16, 8),
            'choropleth_legend': True,
            'choropleth_hue': '',
            'choropleth_cmap': 'Reds',
            'sankey_limits': (0.1, 10),
            'sankey_color': 'black',
            'sankey_figsize': (16, 12),
            'title_fontsize': 20,
            'overlay_type': 'point',
            'overlay_file': '',
            'sankey_plot': False,
            'sankey_scale': '',
            'cartogram_scale': '',
            'cartogram_legend': True,
            'cartogram_cmap': 'Reds',
            'cartogram_hue': '',
            'cartogram_num_legend': 5,
            'path_scale': '',
            'path_legend': True,
            'path_cmap': 'Blues',
            'path_hue': '',
            'path_num_legend': 5,
            'legend_title': '',
            'plot_title': ''
        }

        # when new configuraton is set, update the original one
        conf.update(kwargs)

        # read ploygon file
        try:
            poly_data = self.read_file(file=base_file,
                                       longtitude=conf['file_longtitude'],
                                       latitude=conf['file_latitude'])
        except Exception:
            print(
                'Sorry, this ploygon/base file does not exist, please check the file name'
            )

        if conf['sankey_plot'] == False:
            # read overlay file
            try:
                overlay_data = self.read_file(
                    file=conf['overlay_file'],
                    longtitude=conf['file_longtitude'],
                    latitude=conf['file_latitude'])
            except Exception:
                print(
                    'Sorry, this overlay file does not exist, please check the file name'
                )

            # The polyplot draws polygons on a map, The base file contains the polygons column
            ax = gplt.polyplot(df=poly_data,
                               linewidth=conf['polyplot_linewidth'],
                               facecolor=conf['polyploy_facecolor'],
                               figsize=conf['polyplot_figsize'])
            """ 
            The pointplot is a geospatial scatter plot that represents each observation in your dataset as 
            a single point on a map. It is simple and easily interpretable plot, making it an ideal choice for 
            showing simple pointwise relationships between observations.
            """
            if conf['overlay_type'] == 'point':
                # pointplot
                gplt.pointplot(df=overlay_data,
                               edgecolor=conf['pointplot_edgecolor'],
                               linewidth=conf['pointplot_linewidth'],
                               alpha=conf['pointplot_alpha'],
                               ax=ax)

            elif conf['overlay_type'] == 'kde':
                # kde
                """
                Kernel density estimation is a technique that non-parameterically estimates a distribution function
                for a sample of point observations. KDEs are a popular tool for analyzing data distributions; 
                this plot applies this technique to the geospatial setting.
                """
                ax = gplt.kdeplot(df=overlay_data,
                                  cmap=conf['kde_cmap'],
                                  shade=conf['kde_shade'],
                                  shade_lowest=conf['kde_shade_lowest'],
                                  alpha=conf['kde_alpha'],
                                  ax=ax)

            # Combine point plot and kde plot together
            elif conf['overlay_type'] == 'both':

                # pointplot
                gplt.pointplot(df=overlay_data,
                               edgecolor=conf['pointplot_edgecolor'],
                               linewidth=conf['pointplot_linewidth'],
                               alpha=conf['pointplot_alpha'],
                               ax=ax)

                # kde
                ax = gplt.kdeplot(df=overlay_data,
                                  cmap=conf['kde_cmap'],
                                  shade=conf['kde_shade'],
                                  shade_lowest=conf['kde_shade_lowest'],
                                  alpha=conf['kde_alpha'],
                                  ax=ax)

            elif conf['overlay_type'] == 'choropleth':

                # choropleth plot
                ax = gplt.choropleth(df=overlay_data,
                                     hue=conf['choropleth_hue'],
                                     figsize=conf['choropleth_figsize'],
                                     legend=conf['choropleth_legend'],
                                     cmap='terrain',
                                     ax=ax)

            elif conf['overlay_type'] == 'path':
                """
                A Path diagram visualizes flow through a network. It can be used to show the magnitudes of data moving
                through a system. This plot brings the Sankey diagram into the geospatial context; useful for
                analyzing traffic load a road network.
                """
                scheme = mc.Quantiles(overlay_data[conf['path_scale']],
                                      k=conf['path_num_legend'])
                ax = gplt.sankey(overlay_data,
                                 scale=conf['path_scale'],
                                 hue=conf['path_hue'],
                                 scheme=scheme,
                                 cmap=conf['path_cmap'],
                                 legend=conf['path_legend'],
                                 legend_kwargs={
                                     'loc': 'lower left',
                                     'title': conf['legend_title'],
                                     'title_fontsize': 12,
                                     'fontsize': 12,
                                     'shadow': True,
                                 },
                                 ax=ax)

            elif conf['overlay_type'] == 'cartogram':
                """
                A cartogram distorts (grows or shrinks) polygons on a map according to the magnitude of
                some input data. A basic cartogram specifies data, a projection, and a scale parameter.
                """
                scheme = mc.Quantiles(overlay_data[conf['cartogram_scale']],
                                      k=conf['cartogram_num_legend'])
                ax = gplt.cartogram(
                    df=overlay_data,
                    scale=conf['cartogram_scale'],
                    hue=conf['cartogram_hue'],
                    cmap=conf['cartogram_cmap'],
                    legend=conf['cartogram_legend'],
                    legend_kwargs={
                        'loc': 'lower left',
                        'title': conf['legend_title'],
                        'title_fontsize': 12,
                        'fontsize': 12,
                        'shadow': True,
                    },
                    scheme=scheme,
                    ax=ax,
                )

            # warning for the wrong overlay_type
            else:
                print(
                    "You didn't enter the right overlay_type, please try again"
                )

        # A Sankey diagram visualizes flow through a network.
        if conf['sankey_plot'] == True:
            # sankey plot
            gplt.sankey(df=poly_data,
                        scale=conf['sankey_scale'],
                        limits=conf['sankey_limits'],
                        color=conf['sankey_color'],
                        legend=True,
                        figsize=conf['sankey_figsize'])

        # plot title
        plt.title(conf['plot_title'], fontsize=conf['title_fontsize'])

        # save the plot
        plt.tight_layout()
        plt.savefig(path_img + output_name)

    def route_map(self, file, **kwargs):
        """The route map with multiple stop
        In this function, It can draw the route base on longtitude and latitude.

        Args:
            file: The file name of route_map file, it need to conatain longtitude and latitude.

        """

        conf = {
            'latitude': 'latitude',
            'longtitude': 'longtitude',
            'map_location': [-37.81, 144.955],
            'map_zoom': 15,
            'icon_color': 'green',
            'icon_style': 'plus',
            'line_color': 'red'
        }

        # when new configuraton is set, update the original one
        conf.update(kwargs)

        # read the file and return the pandas dataframe
        try:
            data = self.read_routefile(file=file)
        except Exception:
            print(
                'Sorry, this ploygon/base file does not exist, please check the file name'
            )

        # Get the list of all latitude
        latitude = data[conf['latitude']].to_list()

        # Get the list of all longtitude
        longtitude = data[conf['longtitude']].to_list()

        # create the base map
        map = folium.Map(location=conf['map_location'],
                         zoom_start=conf['map_zoom'])

        # combine the latitude and longtitude become a point
        points = []
        for i in range(len(latitude)):
            points.append([latitude[i], longtitude[i]])

        # add the marker based on the location
        for index, lat in enumerate(latitude):
            folium.Marker([lat, longtitude[index]],
                          popup=('location{}'.format(index)),
                          icon=folium.Icon(
                              color=conf['icon_color'],
                              icon=conf['icon_style'])).add_to(map)

        # connect the line with each points
        folium.PolyLine(points, color=conf['line_color']).add_to(map)

        # display the map
        display(map)


# Data analysis library numpy and pandas
import pandas as pd
import numpy as np

# Data visualization library matplotlib and seaborn
import seaborn as sns
import matplotlib
import matplotlib.pyplot as plt
import matplotlib.font_manager
# Get the file path and file type
import mimetypes
import urllib
import os

#from google.colab import drive	
#drive.mount('/content/drive')

## setting path
# get current path
path_current = os.getcwd()
# the path is where the dataset saved
path_current=path_current.replace('\\', '/')
path = path_current + '/Data/Area/' 
# the "path_img" is the position where final image will be saved
path_img = path_current + '/Images/'


class Area_plot: 
  # read data
  def read_file(self,file):
    file_url = urllib.request.pathname2url(file)
    ftype = mimetypes.guess_type(file_url, strict=True)[0]
    ## read data file according to its formate, default includes three types of files: csv/excel/text
    # read csv format data from the parking dataset
    if 'excel' in ftype:
      # usecols: return a subset of the columns, here choose one column to use in the line chart
      data = pd.read_csv(path+file)
    # read excel format data from the parking dataset
    elif 'sheet' in ftype:
      data = pd.read_excel(path+file)
    # read text format data from the parking dataset
    elif ftype == 'text/plain':
      data = pd.read_csv(path+file, sep="\t")
    else:
      print("File type cannot find!")
    return data

  # Data sorting function 
  # x_col_name:'index' or actual columns name
  # y_col_name: actual y columns name
  # sort the databy using average value,
  def sort(self,data,x_col_name,y_col_name):
    # if the x_col_name is index return sorted y columns name and new index
    if x_col_name[0] == 'index':      
      for i in y_col_name: 
        data.loc['sort',i]=data.loc[:'sort',i].mean()
      newdata=data[y_col_name].sort_values(axis=1,ascending=False,by='sort')      
      y_col_name=newdata.columns.tolist()
      y_col_name=[i for i in y_col_name if i not in x_col_name]
      data.drop(['sort'],inplace=True)
      data.index = pd.RangeIndex(start=0, stop=len(data.index), step=1) 
      return y_col_name,data.index
    else:
      for i in y_col_name:
        data.loc['sort',i]=data.loc[:'sort',i].mean()   
      newdata=pd.concat([data[x_col_name],data[y_col_name].sort_values(axis=1,ascending=False,by='sort')],axis=1)    
      newdata=data[y_col_name].sort_values(axis=1,ascending=False,by='sort')
      y_col_name=newdata.columns.tolist()
      y_col_name=[i for i in y_col_name if i not in x_col_name]
      return y_col_name

  # check the available file name
  # if the input file name already existed then rename to file_1, file_2
  def get_available_name(self,filename):
    n=[1]
    def check_meta(file_name):
      file_name_new=file_name
      if file_name in [os.path.splitext(i)[0] for i in os.listdir(path_img)]:   
          file_name_new=file_name+'_'+str(n[0])
          n[0]+=1
      if file_name_new in [os.path.splitext(i)[0] for i in os.listdir(path_img)]:   
          file_name_new=check_meta(file_name)
      return file_name_new
    available_name=check_meta(filename)
    return available_name

  # file: file name of your data source
  # x_col_name: ['index'] or ['x_column_name_a','x_column_name_b'...]
  # y_col_name: ['y_column_name_a','y_column_name_b'...]
  # plot_type: type of plot: stack ,percentage or simple
  # paper_type : 'single' or 'double'
  def area(self, file,  x_col_name,  y_col_name, plot_type,paper_type,**kwargs):
    
    # Configuration of the line chart
    # plotwidth: width of the plot
    # plotheight: height of the plot
    # backgrid: backgrid of the plot
    # isframe: frame of the plot
    # my_font: the typeface of x, y labels
    # linewidth: linewidth of the lines in the plot
    # gridlinewidth: if backgrid is True, grid linewidth is the line width of background grid
    # labeltext_size: text size of x,y labels
    # x_label: the content of x label
    # y_label: the content of y label
    # labelpad: pad size of label
    # legend_size: size of legend
    # legend_loc: location of legend
    # legend_ncol: the legend columns number
    # title: True or False as options. If it is True, add title for the plot
    # title_pad: if the title is True, modify pad size of title
    # title_size: if the title is True, modify size of title
    # title_loc: if the title is True, modify location of title
    # Stack_colour_set: stack area chart color palette (the number of y columns <7)
    # Area_colour_set: simple area chart color palette (the number of y columns <7)
    # palette: if the number of y columns >7 in all kinds of area chart, use this color palette
    # ticks: True or False as options. If it is True, add ticks of x and y axis.
    # tick_size: size of tick 
    # tick_direction: 'out', 'inout' and 'in' options.
    # y_scale: the extra blank in y axis
    # alpha: transparency of each area
    # arealinewidth: in the simple area chart, the boundary width
    # y_col_num: the critical value of y columns in choosing color palette
    # sort: using sort function to handle data
    # save_image: True or False as options. If it is True, save chart
    # savefig_bbox_inches: Bounding box in inches
    # file_name: the file name in saving image

    single_column_conf={ 'plotwidth':8,#weight
                        'plotheight':6, #height
                        'my_font':'DejaVu Sans',
                        'backgrid':True,
                        'isframe':True,
                        'linewidth':2,
                        'gridlinewidth':0.5,
                        'labeltext_size':15,
                        'x_label':None,
                        'y_label':None,
                        'labelpad':10,
                        'legend_size':10,
                        'legend_loc':'upper right', 
                        'legend_ncol':2,
                        'title':False,
                        'title_pad':8,
                        'title_size':14,
                        'title_loc':'center',     
                        'Stack_colour_set':['006D77','8AA8A1','A1683A','E8998D','D1B490','EE7B30','FFCB77'],
                        'Area_colour_set':[(255/255, 173/255, 173/255),(255/255, 214/255, 165/255),(253/255, 255/255, 182/255),(202/255, 255/255, 191/255),
                                           (155/255, 246/255, 255/255),(160/255, 196/255, 255/255),(255/255, 198/255, 255/255)],
                        'palette':sns.color_palette('Set2'),
                        'ticks':True,
                        'tick_size':14,
                        'tick_direction':'out',
                        'save_image':False,
                        'y_scale':1.20,
                        'alpha':0.7,
                        'arealinewidth':1,
                        'y_col_num':7,
                        'sort':True,
                        'file_name':'area_chart',
                        'savefig_bbox_inches':'tight',
                        'file_name':'area_chart',
                        }  
    double_column_conf={ 'plotwidth':8,#weight
                        'plotheight':6, #height
                        'my_font':'DejaVu Sans',
                        'backgrid':True,
                        'isframe':True,
                        'linewidth':2,
                        'gridlinewidth':0.5,
                        'labeltext_size':25,
                        'x_label':None,
                        'y_label':None,
                        'labelpad':10,
                        'legend_size':10,
                        'legend_loc':'upper right', 
                        'legend_ncol':2,
                        'title':False,
                        'title_pad':10,
                        'title_size':20,
                        'title_loc':'center',     
                        'Stack_colour_set':['006D77','8AA8A1','A1683A','E8998D','D1B490','EE7B30','FFCB77'],
                        'Area_colour_set':[(255/255, 173/255, 173/255),(255/255, 214/255, 165/255),(253/255, 255/255, 182/255),(202/255, 255/255, 191/255),
                                           (155/255, 246/255, 255/255),(160/255, 196/255, 255/255),(255/255, 198/255, 255/255)],
                        'palette':sns.color_palette('Set2'),
                        'ticks':True,
                        'tick_size':20,
                        'tick_direction':'out',
                        'y_scale':1.20,
                        'alpha':0.7,
                        'arealinewidth':1,
                        'y_col_num':7,
                        'sort':True,
                        'save_image':False,
                        'savefig_bbox_inches':'tight',
                        'file_name':'area_chart',
                       
                        }
                         
    if paper_type == 'single':
      conf = single_column_conf
    elif paper_type == 'double':
      conf = double_column_conf                    

    # when new configuraton is set, update the original one
    conf.update(kwargs) 

    # create figure and set figure size  
    fig, ax_left = plt.subplots(figsize = (conf['plotwidth'], conf['plotheight']))

    # background grid setting
    if conf['backgrid'] == True:
      ax_left.grid(linestyle="--", linewidth=conf['gridlinewidth'], color='gray', alpha=0.5)
            
    # read file 
    
    try:
      data = self.read_file(file)
    except Exception:
      print('Sorry, this file does not exist, please check the file name')
 
 
    #plot
    #area chart type setting
    # stack area plot 
    if plot_type == 'Stack':
    # x column is index
      if x_col_name[0] =='index':  
        ydata=[]

        for i in data[y_col_name]:
            ydata.append(data[i].tolist()) 
        # determine the number of y columns is more than critical value or not
        # if yes, using the setted color palette
        if len(y_col_name)<=conf['y_col_num']:        
          ax_left.stackplot(data.index,ydata,colors=conf['Stack_colour_set'],labels=y_col_name,alpha=conf['alpha']) ##
        # if not, using the seaborn color palette
        else:
          ax_left.stackplot(data.index,ydata,colors=conf['palette'],labels=y_col_name,alpha=conf['alpha'])
        # scale y axis to get more blank            
        ax_left_ylim = ax_left.get_ylim()
        ax_left.set_ylim(ax_left_ylim[0],ax_left_ylim[1]*conf['y_scale'])

      # if x colunm is not index
      else:
        ydata=[]
        for i in data[y_col_name]:
            ydata.append(data[i].tolist()) 
        # determine the number of y columns is more than critical value or not
        # if yes, using the setted color palette
        if len(y_col_name)<=conf['y_col_num']:         
          ax_left.stackplot(data[x_col_name[0]],ydata,colors=conf['Stack_colour_set'],labels=y_col_name,alpha=conf['alpha'])
        # if not, using the seaborn color palette  
        else:
          ax_left.stackplot(data[x_col_name[0]],ydata,colors=conf['palette'],labels=y_col_name,alpha=conf['alpha'])
        # scale y axis to get more blank    
        ax_left_ylim = ax_left.get_ylim()
        ax_left.set_ylim(ax_left_ylim[0],ax_left_ylim[1]*conf['y_scale']) 
    
    # percentage area plot
    if plot_type == 'percentage':
      # x column is index
      if x_col_name[0] =='index':
        ydata=[]
        # get the percentange of each part
        data = data[y_col_name].divide(data[y_col_name].sum(axis=1), axis=0)
        for i in data[y_col_name]:
            ydata.append(data[i].tolist()) 
        # determine the number of y columns is more than critical value or not
        # if yes, using the setted color palette
        if len(y_col_name)<=conf['y_col_num']:        
          ax_left.stackplot(data.index,ydata,colors=conf['Stack_colour_set'],labels=y_col_name,alpha=conf['alpha'])
        # if not, using the seaborn color palette  
        else:
          ax_left.stackplot(data.index,ydata,colors=conf['palette'],labels=y_col_name,alpha=conf['alpha'])   
        # scale y axis to get more blank           
        ax_left_ylim = ax_left.get_ylim()
        ax_left.set_ylim(ax_left_ylim[0],ax_left_ylim[1])
      
      # if x colunm is not index
      else:
        ydata=[]
        data = data[y_col_name].divide(data[y_col_name].sum(axis=1), axis=0)

        for i in data[y_col_name]:
          ydata.append(data[i].tolist()) 
        if len(y_col_name)<=conf['y_col_num']:         
          ax_left.stackplot(data[x_col_name[0]],ydata,colors=conf['Stack_colour_set'],labels=y_col_name,alpha=conf['alpha'])
            
        else:
          ax_left.stackplot(data[x_col_name[0]],ydata,colors=conf['palette'],labels=y_col_name,alpha=conf['alpha'])
        ax_left_ylim = ax_left.get_ylim()
        ax_left.set_ylim(ax_left_ylim[0],ax_left_ylim[1]) 
    
    #simple area chart
    if plot_type == 'Simple':
     
      # x column is index      
      if x_col_name[0] =='index':
        # data handling by using sort function
        if conf['sort'] == True:
          # if the input x coluns is index, the function return the sorted y columns and new index
          y_col_name=self.sort(data,x_col_name,y_col_name)[0]
          data.index=self.sort(data,x_col_name,y_col_name)[1]
        
        for i in range(0,len(y_col_name)):
          # determine the number of y columns is more than critical value or not
          # if yes, using the setted color palette
          if len(y_col_name)<=conf['y_col_num']:                     
            ax_left.fill_between(data.index, data[y_col_name[i]], color=conf['Area_colour_set'][i],
                 alpha=0.7, label=y_col_name[i])
          # if not, using the seaborn color palette    
          else:
            ax_left.fill_between(data.index, data[y_col_name[i]], color=conf['palette'][i],
                 alpha=0.7, label=y_col_name[i])
          # draw the bounday line   
          ax_left.plot(data.index, data[y_col_name[i]], color="Black", alpha=conf['alpha'], linewidth=conf['arealinewidth'])
        # scale y axis to get more blank 
        ax_left_ylim = ax_left.get_ylim()         
        ax_left.set_ylim(ax_left_ylim[0],ax_left_ylim[1]*conf['y_scale'])

      # x colunm is not index 
      else:
        #data handling by using sort function
        if conf['sort'] == True:
          # if the x column name is actual columns name, the function return sorted y columns name
          y_col_name=self.sort(data,x_col_name,y_col_name)
        
        for i in range(0,len(y_col_name)):
          if len(y_col_name)<=conf['y_col_num']: 
            ax_left.fill_between(data[x_col_name[0]], data[y_col_name[i]], color=conf['Area_colour_set'][i],
                  alpha=conf['alpha'], label=y_col_name[i])
          else:
            ax_left.fill_between(data[x_col_name[0]], data[y_col_name[i]], color=conf['palette'][i],
                  alpha=conf['alpha'], label=y_col_name[i])
          ax_left.plot(data[x_col_name[0]], data[y_col_name[i]], color="Black", alpha=conf['alpha'], linewidth=conf['arealinewidth'])
        ax_left_ylim = ax_left.get_ylim()          
        ax_left.set_ylim(ax_left_ylim[0],ax_left_ylim[1]*conf['y_scale'])
    
    #set ticks size
    for tick in ax_left.xaxis.get_major_ticks():
      tick.label.set_fontsize(conf['tick_size'])
    for tick in ax_left.yaxis.get_major_ticks():
      tick.label.set_fontsize(conf['tick_size'])

    # if False, top and right borders removing    
    if conf['isframe'] == False:
        ax.spines['top'].set_visible(False)
        ax.spines['right'].set_visible(False)
        
    # x, y axis setting
    ax_left.set_xlabel(conf['x_label'], fontproperties=conf['my_font'], fontsize=conf['labeltext_size'], labelpad=conf['labelpad'])
    ax_left.set_ylabel(conf['y_label'], fontproperties=conf['my_font'], fontsize=conf['labeltext_size'], labelpad=conf['labelpad'])

    # set x, y tick's direction, default:out, can be set to in,out,inout
    if conf['tick_direction'] == 'in':
      matplotlib.rcParams['xtick.direction'] = 'in'
      matplotlib.rcParams['ytick.direction'] = 'in'
    elif conf['tick_direction'] == 'inout':
      matplotlib.rcParams['xtick.direction'] = 'inout'
      matplotlib.rcParams['ytick.direction'] = 'inout'
    elif conf['tick_direction'] == 'out':
      matplotlib.rcParams['xtick.direction'] = 'out'
      matplotlib.rcParams['ytick.direction'] = 'out'

    #legend setting
    # show in 2 line, location: upper right 
    ax_left.legend(ncol=conf['legend_ncol'],loc=conf['legend_loc'], fontsize=conf['legend_size']) #conf
    #title setting
    if conf['title'] == False:
      pass
    else:
      ax_left.set_title(conf['title'], fontsize=conf['title_size'], loc=conf['title_loc'], pad=conf['title_pad'])
    #save image
    if conf['save_image'] == True:
      file_name=conf['file_name']
      # use function to get available file name
      file_newname = self.get_available_name(file_name)
      plt.savefig(path_img+file_newname, bbox_inches=conf['savefig_bbox_inches'],dpi=600,format='jpg') 
     
    # showing the image
    plt.show()


# import python data analysis library numpy and pandas 
import pandas as pd
import numpy as np

# import data visualization library matplotlib and seaborn
import matplotlib.pyplot as plt
import matplotlib.ticker as ticker
import matplotlib.font_manager
import seaborn as sns
from seaborn.utils import relative_luminance

# import module to read files
import mimetypes
import urllib
import os

# import google colab to use google colab as editor
# if you use other editor, do not need to import google.colab here
#from google.colab import drive
#drive.mount('/content/drive', force_remount = True)
## setting path
# get current path
path_current = os.getcwd()

path_current=path_current.replace('\\', '/')
# the path is where the dataset saved
path = path_current + '/Data/Bar/' 
# the "path_img" is the position where final image will be saved
path_img = path_current + '/Images/'
class bar_chart:
    def __init__(self, y_series = '',  label_list = '', 
               x_series = ''):
      self.y_series = y_series
      self.x_series = x_series

    ## read file function: read three kinds of format file csv/excel/text
    # if you have other format of file, please change the function manually
    # file: str, filename (e.g.'Vertical_Bar.txt')
    def read_file(self,file):
      file_url = urllib.request.pathname2url(file)
      ftype = mimetypes.guess_type(file_url, strict = True)[0]

      ## read data file according to format, default file types: csv/excel/text
      # read csv format data
      if 'excel' in ftype:
          data = pd.read_csv(path + file)
      # read excel format data
      elif 'sheet' in ftype:
          data = pd.read_excel(path + file)
      # read text format data
      elif ftype == 'text/plain':
          data = pd.read_csv(path + file, sep="\t")
      else:
          print("Cannot read file, change file type.")
      return data

    def BarColor(self):
      ## create example figure
      fig = plt.figure(figsize = (conf['plotwidth'], conf['plotheight']))
      ax = fig.add_subplot(111)

      ## format ticks
      # MultipleLocator: locates ticks at a multiple of the number provided
      ax.xaxis.set_major_locator(ticker.MultipleLocator(1))
      ax.yaxis.set_major_locator(ticker.MultipleLocator(0.5))

      ## set the ticks position
      ax.xaxis.set_ticks_position('bottom')
      # {'top', 'bottom', 'both', 'default', 'none'}

      ## draw example bar plot
      for i in range(len(conf['colors'])):
          ax.bar(i, 3, color= conf['colors'][i], edgecolor='black')

      ## show color match
      plt.show()

    def BarPattern(self):
      ## create example figure
      fig = plt.figure(figsize = (conf['plotwidth'], conf['plotheight']))
      ax = fig.add_subplot(111)

      ## format ticks
      # MultipleLocator: locates ticks at a multiple of the number you provide
      ax.xaxis.set_major_locator(ticker.MultipleLocator(1))
      ax.yaxis.set_major_locator(ticker.MultipleLocator(0.5))

      ## set the ticks position
      ax.xaxis.set_ticks_position('bottom')
      # {'top', 'bottom', 'both', 'default', 'none'}

      ## draw example bar plot
      for i in range(len(conf['patterns'])):
          ax.bar(i, 3, color='white', edgecolor='black', hatch=conf['patterns'][i])

      ## show hatch patterns
      plt.show()

    def Bar(self, file,  x_col_name,  y_col_name, x_label, y_label,  paper_type, direction,  **kwargs):


             
      single_column_conf = {'isframe':False,
                  'type': 'single',
                  'title':'',
                  'title_pad':10,
                  'title_size':20,
                  'title_loc':'center',
                  'legend_label':'',
                  'plotwidth':10,
                  'plothight':6,
                  'line_width':1,
                  'valueformat':'{:.0f}',
                  'bar_width':0.8,
                  'xax_length':1,
                  'yax_length':1,
                  'valuepad':1,
                  'err_series':'',
                  'xtick_loc':1,
                  'ytick_loc':5,
                  'tick_labels':'',
                  'tick_direction':'out',
                  'fontfamily':'sans',
                  'axfont_size':17,
                  'valuefont_size':13,
                  'tickfont_size':13,
                  'legfont_size':14,
                  'labelpad':10,
                  'fontsize':18,
                  'legendcol':1,
                  'legend_loc':'upper right',
                  'gridline_width':0.2,
                  'colors':'1' * 50,
                  'bar_color':['#7173A9','#E8AA78','#E2918F','#629DDD',
                         '#A4BF7F','#A5D7D8','#A48BC1','#F4D470'],
                  'axis_color':'0',
                  'grid_color':'grey',
                  'edge_color':'0',
                  'font_color':'0',
                  'patterns':' ' * 50,
                  'bar_pattern':'0',
                  'eline_width':0.7,
                  'cap_size':0,
                  'cap_thick':1,
                  'is_save_fig':False,
                  'save_image':'bar.pdf',
                  'show_value':True}

      #double_column_conf                 
                  
      if paper_type == 'single':
        conf = single_column_conf
      elif paper_type == 'double':
        conf = double_column_conf 
    
      # when new configuraton is set, update the original one
      conf.update(kwargs)
                      
      # conf['err_series']              
  
      ## set font dictionary
      font_dict = {'family': conf['fontfamily'],
              'color': conf['font_color']}

      ## read file
      data = self.read_file(file)

      ## create figure and set figure size  
      fig, ax = plt.subplots(figsize = (conf['plotwidth'], conf['plothight']))


      ## add axes and format axes
      ax = fig.add_axes([0, 0, conf['xax_length'], conf['yax_length']])
      # The dimensions [left, bottom, width, height] of the new axes (range 0 to 1)
      # All quantities are in fractions of figure width and height.

      ## plot size changes according to figure size and the width,height of axes 
      
      if conf['type']=='single':

        ## draw vertical bar plot
        # edgecolor: The colors of the bar edges.
        # linewidth: Width of the bar edge(s). If 0, don't draw edges.
        # color: The colors of the bar faces. Default setting 'C0'.
        categories = []
        ticklabel_list = [0]
        for x, v in enumerate(list(data[x_col_name])):
          categories.append(x + 1.0)
          ticklabel_list.append(v)
        # choose the direction of bar: 'vertical' or 'horizontal'
        if direction == 'vertical':
          rectobj = ax.bar(x=categories, height=data[y_col_name], width=conf['bar_width'], 
                    data=data, label=conf['legend_label'],edgecolor=conf['edge_color'], 
                    linewidth=conf['line_width'], color=conf['bar_color'], hatch=conf['bar_pattern'])
        elif direction == 'horizontal':
          rectobj = ax.barh(y=data[y_col_name], height=conf['bar_width'], width=data[x_col_name], 
                    data=data, label=conf['legend_label'], edgecolor=conf['edge_color'], 
                    linewidth=conf['line_width'], color=conf['bar_color'], hatch=conf['bar_pattern'])

        ## format axes spines
        ## remove top and right spines
        ## set color of left and bottom axes spines
        # if False, top and right borders removing
        if conf['isframe'] == False:
          ax.spines['top'].set_visible(False)
          ax.spines['right'].set_visible(False)

        ## format ticks
        # MultipleLocator: locates ticks at a multiple of the number you provide
        ax.xaxis.set_major_locator(ticker.MultipleLocator(conf['xtick_loc']))
        ax.yaxis.set_major_locator(ticker.MultipleLocator(conf['ytick_loc']))

        ## set x, y tick's direction, default:out, can be set to in,out,inout
        if conf['tick_direction'] == 'in':
          matplotlib.rcParams['xtick.direction'] = 'in'
          matplotlib.rcParams['ytick.direction'] = 'in'
        elif conf['tick_direction'] == 'inout':
          matplotlib.rcParams['xtick.direction'] = 'inout'
          matplotlib.rcParams['ytick.direction'] = 'inout'
        elif conf['tick_direction'] == 'out':
          matplotlib.rcParams['xtick.direction'] = 'out'
          matplotlib.rcParams['ytick.direction'] = 'out'

        ## format the tick color and tick labels
        # color: the color of ticks
        # labelsize: the font size of tick labels
        # labelcolor: the color of tick labels
        ax.tick_params(color = conf['axis_color'], labelsize = conf['tickfont_size'], labelcolor = conf['font_color'])
        
        if direction == 'vertical':
          if conf['tick_labels'] == '':
            ax.set_xticklabels(ticklabel_list)
          else:
            ax.set_xticklabels(conf['tick_labels'])
          ## add background grid
          ax.yaxis.grid(linestyle="--",color='gray', linewidth = conf['gridline_width'], alpha = 0.5)
          # alpha: adjust the transparency
        
        elif direction == 'horizontal':
          if conf['tick_labels']=='':
            ax.set_yticklabels(ticklabel_list)
          else:
            ax.set_yticklabels(conf['tick_labels'])
          ## add background grid
          ax.xaxis.grid(linestyle="--",color='gray', linewidth = conf['gridline_width'], alpha = 0.5) 
          # alpha: adjust the transparency

      elif conf['type']=='group':
        
        
        # number of groups
        

        if direction == 'vertical':      
          # set location of bars on x-axis
          # initialise list
          n_groups = len(y_col_name)
          xloc_list = [0]*n_groups
          xloc = np.arange(len(data[x_col_name]))
          for i in range(n_groups):
            xloc_list[i] = [x + i*conf['bar_width'] for x in xloc]

          rectobj_list = [0] * n_groups

          ## draw bars
          for i in range(n_groups):
              if conf['err_series'] == '':
                rectobj_list[i] = ax.bar(x=xloc_list[i], height=data[y_col_name[i]], width=conf['bar_width'],
                              color = conf['bar_color'][i], edgecolor = conf['edge_color'], 
                              linewidth = conf['line_width'],label = conf['legend_label'][i])
              else: 
                err =  data[conf['err_series'][i]].tolist()
                rectobj_list[i] = ax.bar(x=xloc_list[i], height=data[y_col_name[i]], width=conf['bar_width'], 
                              yerr=err, color=conf['bar_color'][i], 
                              edgecolor = conf['edge_color'], linewidth = conf['line_width'], 
                              label = conf['legend_label'][i], capsize = conf['cap_size'],
                              error_kw = {'elinewidth': conf['eline_width'], 'capthick': conf['eline_width']})

        elif direction == 'horizontal':  
          ## set locations for the bars on y axis
          # initialise list
          n_groups = len(x_col_name)
          yloc_list=[0]*n_groups
          yloc = np.arange(len(data[y_col_name]))
          for i in range(n_groups):
            yloc_list[i]=[y + i*conf['bar_width'] for y in yloc]

          rectobj_list = [0] * n_groups

          ## draw grouped bars 
          for i in range(n_groups):
              if conf['err_series']=='':
                rectobj_list[i] = ax.barh(y=yloc_list[i], height=conf['bar_width'], width=data[x_col_name[i]], 
                              color=conf['bar_color'][i], edgecolor=conf['edge_color'], linewidth=conf['line_width'], 
                              label=conf['legend_label'][i])    
              else: 
                err=data[conf['err_series'][i]].tolist()
                rectobj_list[i] = ax.barh(y=yloc_list[i], height=conf['bar_width'], width=data[x_col_name[i]], 
                              xerr = err, color = conf['bar_color'][i], edgecolor = conf['edge_color'], 
                              linewidth = conf['line_width'], label=conf['legend_label'][i],
                              capsize = conf['cap_size'], error_kw = {'elinewidth': conf['eline_width'],'capthick': conf['eline_width']})



        ## format axes spines
        ## remove top and right spines
        ## set color of left and bottom axes spines

        if conf['isframe'] == False:
          ax.spines['top'].set_visible(False)
          ax.spines['right'].set_visible(False)


        ## format ticks
        # MultipleLocator: locates ticks at a multiple of the number you provide
        ax.xaxis.set_major_locator(ticker.MultipleLocator(conf['xtick_loc']))
        ax.yaxis.set_major_locator(ticker.MultipleLocator(conf['ytick_loc']))
        
        if direction == 'vertical':
          ## adjust the location of xticks label
          if conf['tick_labels'] == '':
            plt.xticks([x + conf['bar_width']*0.5*(n_groups-1) for x in range(len(data[x_col_name]))], data[x_col_name].tolist())
          else:  
            plt.xticks([x + conf['bar_width'] for x in range(n_groups)], conf['tick_labels'])
          ## format tick color and tick labels
          # color: the color of ticks
          # labelsize: the font size of tick labels
          # labelcolor: the color of tick labels
          ax.tick_params(color = conf['axis_color'], labelsize = conf['tickfont_size'], labelcolor = conf['font_color'])
          ## add background grid
          ax.yaxis.grid(linestyle="--",color='gray', linewidth = conf['gridline_width'], alpha = 0.5) # alpha: adjust the transparency          

        elif direction=='horizontal':
          ## format tick color and tick labels
          if conf['tick_labels'] == '':
            # adjust the location of yticks label
            plt.yticks([y + conf['bar_width']*0.5*(n_groups-1) for y in range(len(data[y_col_name]))], data[y_col_name].tolist())
          else: 
            # adjust the location of yticks label 
            plt.yticks([y + conf['bar_width']*0.5*(n_groups-1) for y in range(len(data[y_col_name]))], conf['tick_labels'])
          # color: the color of ticks
          # labelsize: the font size of tick labels
          # labelcolor: the color of tick labels
          ax.tick_params(color = conf['axis_color'], labelsize = conf['tickfont_size'], labelcolor = conf['font_color'])
          ## add background grid
          ax.xaxis.grid(linestyle="--",color='gray', linewidth = conf['gridline_width'], alpha = 0.5) # alpha: adjust the transparency
      
      elif conf['type']=='stack':
        if direction == 'vertical':
          n_groups = len(y_col_name)
          
          ## set locations for the bars on x axis(vertical bar)
          xloc = np.arange(len(data[x_col_name]))  

          bottom_value_list = [[0]] * n_groups
          rectobj_list = [0] * n_groups
          
          ## draw grouped bar plot
          for i in (range(n_groups)):
              if conf['err_series'] == '':
                  err = None
              else:
                  err = data[conf['err_series'][i]].tolist()
              if i == 0:
                  bottom_value_list[0] = 0 * n_groups
              else:
                  bottom_value_list[i] = np.array(bottom_value_list[i-1]) + np.array(data[y_col_name[i-1]])            

              rectobj_list[i] = ax.bar(x=xloc, height=data[y_col_name[i]], width=conf['bar_width'],
                            yerr=err, bottom=bottom_value_list[i], color=conf['bar_color'][i], 
                            edgecolor = conf['edge_color'], linewidth = conf['line_width'],
                            label = conf['legend_label'][i], capsize = conf['cap_size'],
                            error_kw = {'elinewidth': conf['eline_width'], 'capthick': conf['eline_width']})
        elif direction == 'horizontal':
          n_groups = len(x_col_name)
          
          ## set locations for the bars on y axis(horizontal bar)
          yloc = np.arange(len(data[y_col_name].tolist()))
          
          bottom_value_list = [[0]] * n_groups
          rectobj_list = [0] * n_groups
          ## draw grouped bar plot
          for i in (range(n_groups)):
              if conf['err_series'] == '':
                  err = None
              else:
                  err = data[conf['err_series'][i]].tolist()
              if i == 0:
                  bottom_value_list[0] = 0 * n_groups
              else:
                  bottom_value_list[i] = np.array(bottom_value_list[i-1]) + np.array(data[x_col_name[i-1]])            

              rectobj_list[i] = ax.barh(y=yloc, height=conf['bar_width'], width=data[x_col_name[i]], 
                            xerr=err, left=bottom_value_list[i], color=conf['bar_color'][i],  
                            edgecolor = conf['edge_color'], linewidth = conf['line_width'],
                            label = conf['legend_label'][i], capsize = conf['cap_size'],
                            error_kw = {'elinewidth': conf['eline_width'], 'capthick': conf['eline_width']})
        


        ## format ticks
        # MultipleLocator: locates ticks at a multiple of the number you provide
        ax.xaxis.set_major_locator(ticker.MultipleLocator(conf['xtick_loc']))
        ax.yaxis.set_major_locator(ticker.MultipleLocator(conf['ytick_loc']))  

        ## set x, y tick's direction, default:out, can be set to in,out,inout
        if conf['tick_direction'] == 'in':
          matplotlib.rcParams['xtick.direction'] = 'in'
          matplotlib.rcParams['ytick.direction'] = 'in'
        elif conf['tick_direction'] == 'inout':
          matplotlib.rcParams['xtick.direction'] = 'inout'
          matplotlib.rcParams['ytick.direction'] = 'inout'
        elif conf['tick_direction'] == 'out':
          matplotlib.rcParams['xtick.direction'] = 'out'
          matplotlib.rcParams['ytick.direction'] = 'out'

        ## format the tick color and tick labels
        # color: the color of ticks
        # labelsize: the font size of tick labels
        # labelcolor: the color of tick labels
        ax.tick_params(color = conf['axis_color'], labelsize = conf['tickfont_size'], labelcolor = conf['font_color'])
        
        if direction == 'vertical':
          if conf['tick_labels'] == '':
            plt.xticks([x  for x in range(len(data[x_col_name]))], data[x_col_name].tolist())
          else:
            plt.xticks([x  for x in range(len(data[x_col_name]))], conf['tick_labels'])
          ## add background grid
          ax.yaxis.grid(linestyle="--",color='gray', linewidth = conf['gridline_width'], alpha = 0.5)
          # alpha: adjust the transparency
        
        elif direction == 'horizontal':
          if conf['tick_labels']=='':
            plt.yticks([y  for y in range(len(data[y_col_name]))], data[y_col_name].tolist())
          else:
            plt.yticks([y  for y in range(len(data[x_col_name]))], conf['tick_labels'])
          ## add background grid
          ax.xaxis.grid(linestyle="--",color='gray', linewidth = conf['gridline_width'], alpha = 0.5) 
          # alpha: adjust the transparency
      
      def autolabel(rects):

          if conf['err_series'] == '':
            if direction == 'vertical':
              ha='center'
              va='bottom'
            elif direction == 'horizontal':
              ha='left'
              va='center'
          else:
            if direction == 'vertical':
              ha='center'
              va='bottom'
            elif direction == 'horizontal':
              ha='left'
              va='top'            
          if direction == 'vertical':
            # Get y-axis height to calculate label position from.
            (y_bottom, y_top) = ax.get_ylim()
            y_height = y_top - y_bottom
            for rect in rects:
                height = rect.get_height()
                # Fraction of axis height taken up by this rectangle
                p_height = (height / y_height)
                # If we can fit the label above the column, do that;
                # otherwise, put it inside the column.

                label_position = height + (y_height * 0.01)

                ax.text(rect.get_x() + rect.get_width() / 2., label_position,
                      round(height, 2), ha=ha, va=va)
          else:
            # Get y-axis height to calculate label position from.
            (x_left, x_right) = ax.get_xlim()
            x_width = x_right - x_left
            for rect in rects:
                width = rect.get_width()
                # Fraction of axis height taken up by this rectangle
                p_width = (width / x_width)
                # If we can fit the label above the column, do that;
                # otherwise, put it inside the column.

                label_position = width + (x_width * 0.01)

                ax.text(label_position, rect.get_y() + rect.get_height()/2.,
                      round(width, 2),  ha=ha, va=va)

      ## format axes spines
      ## remove top and right spines
      ## set color of left and bottom axes spines
      # if False, top and right borders removing
      if conf['isframe'] == False:
        ax.spines['top'].set_visible(False)
        ax.spines['right'].set_visible(False)
      
      if conf['show_value'] == True:     
        if conf['err_series'] == '':
          if  conf['type']=='single':
            autolabel(rectobj)
          else:  
            for i in range(n_groups):
              autolabel(rectobj_list[i])    
        else:
          for i in (range(n_groups)):
            if direction == 'vertical':
              autolabel(rectobj_list[i])
            elif direction == 'horizontal':
              autolabel(rectobj_list[i]) 
      
      ## format title
      if conf['title'] == '':
        pass
      else:
        ax.set_title(conf['title'], fontsize=conf['title_size'], loc=conf['title_loc'], pad=conf['title_pad']) 
            
             
      ## create legend
      # nocl:  integer, the number of columns that the legend has
      # fontsize: the font size of legend text
      # loc: str or pair of floats, the location of the legend (default 'best' or 0)
      plt.legend(ncol = conf['legendcol'], fontsize = conf['legfont_size'], loc = conf['legend_loc'])

      ## format x,y labels
      plt.xlabel(x_label, fontsize = conf['axfont_size'], labelpad = conf['labelpad'], color = conf['font_color'], fontdict=font_dict)
      plt.ylabel(y_label, fontsize = conf['axfont_size'], labelpad = conf['labelpad'], color = conf['font_color'], fontdict=font_dict)
      # labelpad: Spacing in points between the label and the x-axis

      ## save image as pdf to path folder
      # bbox in inches, only the given portion of the figure is saved,
      # figure out the tight bbox of the figure
      if conf['is_save_fig'] == True:
          plt.savefig(path_img + conf['save_image'], bbox_inches = 'tight')
      
      plt.show()


## import python data analysis library 
import pandas as pd

## import data visualization library matplotlib and seaborn
import seaborn as sns
import matplotlib
import matplotlib.pyplot as plt
from matplotlib.ticker import MultipleLocator, FormatStrFormatter, AutoMinorLocator, FuncFormatter
from matplotlib.ticker import AutoMinorLocator, MultipleLocator, FuncFormatter
from matplotlib import rc
from mpl_toolkits.axes_grid.inset_locator import (inset_axes, InsetPosition, mark_inset, zoomed_inset_axes)
import matplotlib.cm as cm
import numpy as np
import matplotlib.patches as mpatches
from matplotlib.spines import Spine

## import module to read files
import mimetypes
import urllib
import os
path_current = os.getcwd()

path_current=path_current.replace('\\', '/')
# the path is where the dataset saved
path = path_current + '/Data/Rad/' 
# the "path_img" is the position where final image will be saved
path_img = path_current + '/Images/'
class Radar:
    # read data
    def read_file(self, file):
        """Read different types of files and return pandas dataframe.

        This function can transform multiple file type such as csv/excel/text into
        a pandas dataframe. User need to input the filename such as: 'file.csv'.
        If the file type cannot be find or not support it, it will return the message.

        Args:
            file: filename of user data source

        Returns:
            data : A pandas dataframe

        """
        # Get the file URL
        file_url = urllib.request.pathname2url(file)  # try catch
        # use mimetypes package to guess the file type
        # For example: 'file.csv' will return 'excel'
        ftype = mimetypes.guess_type(file_url, strict=True)[0]
        ## read data file according to its formate, default includes three types of files: csv/excel/text
        # read csv format data
        if 'excel' in ftype:
            # usecols: return a subset of the columns, here choose one column to use in the line chart
            data = pd.read_csv(path + file)
        # read excel format data
        elif 'sheet' in ftype:
            data = pd.read_excel(path + file)
        # read text format data
        elif ftype == 'text/plain':
            data = pd.read_csv(path + file, sep="\t")
        else:
            print("File type cannot find!")
        return data

    def radar_plot(self, file, index_col, category_col, y_col_list, y_lim,output_name,
                   **kwargs):
        """Radar chart with selected overlay

        This function will convert the user's incoming data set into DataFrame and then draw
        the radar Chart using Matplotlib. The radar chart can show category, y, overlay.

        To use this function, User need a dataset with three variable columns. The three variables are
        index_col, category_col and y_col_list. The index column usually the name of that row such as 'player,
        The category_col is list of category and the y_col_list is the list of y value need to present.

        In our example, The index_col is player, The category_col is 
        ['Pace','Shooting','Passing','Dribbling','Defending','Physical'],
        and the y_col_list is ['Cristiano Ronaldo'].

        Args:
            file: file name of your data source, it support csv/excel,text.for example:'new_file.csv'
            index_col: The index column. Normally it should be the name of the content
            category_col: input list of attributes for the radar plot
            y_col_name: input list of items
            y_lim: The Y lim value

        """

        conf = {
            'figsize': 
            (6, 4),
            'overlay':
            False,
            'linewidth':
            2,
            'gridlinewidth':
            0.8,
            'labeltext_size':
            20,
            'labelpad':
            10,
            'legend_size':
            10,
            'legend_loc':
            'upper right',
            'title':
            False,
            'title_pad':
            10,
            'title_size':
            15,
            'title_loc':
            'center',
            'title_height':
            1.1,
            'xtick_fontsize':
            15,
            'ytick_fontsize':
            15,
            'ytick_color':
            'blue',
            'xtick_color':
            'black',
            'alpha':
            0.3,
            'legendbox_loc': (1.1, 1.0),
            'plot_title':
            ''
        }

        # when new configuraton is set, update the original one
        conf.update(kwargs)

        # Input the valid filename and return the pandas dataframe for drawing
        data = self.read_file(file)

        # set the index column
        data = data.set_index(index_col)
        # set the selected category from user input
        data = data[category_col]
        # set the y column
        data = data.loc[y_col_list]

        # create the close shape list
        close_shape = []

        # plot height and width
        fig = plt.figure(figsize=conf['figsize'])
        # polar plot
        ax = fig.add_subplot(111, polar=True)

        # radar plot
        for i in data.values:
            # link the line become a close shape radar plot
            close_shape = np.concatenate((i, [i[0]]))
            # calculate the angel of label placement
            label_placement = np.linspace(start=0,
                                          stop=2 * np.pi,
                                          num=len(close_shape))
            ax.plot(label_placement, close_shape)

            # The overlay background
            if conf['overlay'] == True:
                ax.fill(label_placement, close_shape, alpha=conf['alpha'])

        # apply the labels and line into the right place
        lines, labels = plt.thetagrids(np.degrees(label_placement),
                                       labels=category_col)

        # add the legend, legend will show the y values
        ax.legend(labels=y_col_list,
                  bbox_to_anchor=conf['legendbox_loc'],
                  fontsize=conf['legend_size'])

        # x tricks and y ticks color and fontsize
        plt.yticks(color=conf['ytick_color'], fontsize=conf['ytick_fontsize'])
        plt.xticks(color=conf['xtick_color'], fontsize=conf['xtick_fontsize'])

        # Y lim value from user input for accurcy
        plt.ylim(y_lim)

        # plot title
        plt.title(conf['plot_title'],
                  fontsize=conf['title_size'],
                  y=conf['title_height'])
        
        

        # xtrick alignment, It can automatict adjust x tricks location, let the x tricks lables outside of the plot
        for theta, label in zip(ax.get_xticks(), ax.get_xticklabels()):
            theta = theta * ax.get_theta_direction() + ax.get_theta_offset()
            theta = np.pi / 2 - theta
            y, x = np.cos(theta), np.sin(theta)
            if x >= 0.1:
                label.set_horizontalalignment('left')
            if x <= -0.1:
                label.set_horizontalalignment('right')
            if y >= 0.5:
                label.set_verticalalignment('bottom')
            if y <= -0.5:
                label.set_verticalalignment('top')
        # save the plot
        plt.tight_layout()
        plt.savefig(path_img + output_name)

    def spider_plot(self, file, index_col, category_col, y_col_list, y_lim,output_name,
                    **kwargs):
        """Spider chart with selected overlay

        This function will convert the user's incoming data set into DataFrame and then draw
        the spider Chart using Matplotlib. The radar chart can show category, y, overlay.

        To use this function, User need a dataset with three variable columns. The three variables are
        index_col, category_col and y_col_list. The index column usually the name of that row such as 'player,
        The category_col is list of category and the y_col_list is the list of y value need to present.

        In our example, The index_col is status, The category_col is 
        ['blood sugar','blood pressure','blood oxygen','blood fat',' heart rate'],
        and the y_col_list is ['Before','After'].

        Args:
            file: file name of your data source, it support csv/excel,text.for example:'new_file.csv'
            index_col: The index column. Normally it should be the name of the content
            category_col: input list of attributes for the radar plot
            y_col_name: input list of items
            y_lim: The Y lim value

        """

        conf = {
            'figsize':
            (6, 4),
            'overlay':
            False,
            'linewidth':
            2,
            'gridline_width':
            0.8,
            'gridline_color':
            'grey',
            'gridline_style':
            '-',
            'labeltext_size':
            20,
            'labelpad':
            10,
            'legend_size':
            10,
            'legend_loc':
            'upper right',
            'title':
            False,
            'title_pad':
            10,
            'title_size':
            15,
            'title_loc':
            'center',
            'title_height':
            1.1,
            'xtick_fontsize':
            15,
            'ytick_fontsize':
            15,
            'ytick_color':
            'blue',
            'xtick_color':
            'black',
            'alpha':
            0.3,
            'legendbox_loc': (1.3, 1.0),
            'plot_title':
            ''
        }

        # when new configuraton is set, update the original one
        conf.update(kwargs)

        # read file
        data = self.read_file(file)

        # set the index column
        data = data.set_index(index_col)
        # set the selected category from user input
        data = data[category_col]
        # set the y column
        data = data.loc[y_col_list]
        # create the close shape list
        close_shape = []
        # plot height and width
        fig = plt.figure(figsize=conf['figsize'])
        # polar plot
        ax = fig.add_subplot(111, polar=True)

        # spider plot
        for i in data.values:
            # link the line become a close shape radar plot
            close_shape = np.concatenate((i, [i[0]]))
            # calculate the angel of label placement
            label_placement = np.linspace(start=0,
                                          stop=2 * np.pi,
                                          num=len(close_shape))
            ax.plot(label_placement, close_shape)

            # The overlay background
            if conf['overlay'] == True:
                ax.fill(label_placement, close_shape, alpha=conf['alpha'])

            # change the gridline become spider plot
            pl = ax.yaxis.get_gridlines()
            for line in pl:
                line.get_path()._interpolation_steps = len(close_shape)

        # adjust the gridline color,linewidth and linestyle
        ax.grid(color=conf['gridline_color'],
                linestyle=conf['gridline_style'],
                linewidth=conf['gridline_width'])

        # apply the labels and line into the right place
        lines, labels = plt.thetagrids(np.degrees(label_placement),
                                       labels=category_col)

        # add the legend, legend will show the y values
        ax.legend(labels=y_col_list,
                  bbox_to_anchor=conf['legendbox_loc'],
                  fontsize=conf['legend_size'])

        # x tricks and y ticks color and fontsize
        plt.yticks(color=conf['ytick_color'], fontsize=conf['ytick_fontsize'])
        plt.xticks(color=conf['xtick_color'], fontsize=conf['xtick_fontsize'])

        # Y lim value from user input for accurcy
        plt.ylim(y_lim)

        # save the plot
        plt.savefig(path_img + output_name)

        # plot title
        plt.title(conf['plot_title'],
                  fontsize=conf['title_size'],
                  y=conf['title_height'])

        # xtrick alignment, It can automatict adjust x tricks location, let the x tricks lables outside of the plot
        for theta, label in zip(ax.get_xticks(), ax.get_xticklabels()):
            theta = theta * ax.get_theta_direction() + ax.get_theta_offset()
            theta = np.pi / 2 - theta
            y, x = np.cos(theta), np.sin(theta)
            if x >= 0.1:
                label.set_horizontalalignment('left')
            if x <= -0.1:
                label.set_horizontalalignment('right')
            if y >= 0.5:
                label.set_verticalalignment('bottom')
            if y <= -0.5:
                label.set_verticalalignment('top')

        # save the plot
        plt.tight_layout()
        plt.savefig(path_img + output_name)



# Data analysis library numpy and pandas
import pandas as pd
import numpy as np

# Data visualization library matplotlib and seaborn
import seaborn as sns
import matplotlib
import matplotlib.pyplot as plt
import matplotlib.font_manager
import matplotlib.patches as mpatches

import mimetypes
import urllib
import os

#from google.colab import drive	
#drive.mount('/content/drive')

## setting path
# get current path
path_current = os.getcwd()

path_current=path_current.replace('\\', '/')
# the path is where the dataset saved
path = path_current + '/Data/Vio/' 
# the "path_img" is the position where final image will be saved
path_img = path_current + '/Images/'

class Violin_plot: 
  # read data
  def read_file(self,file):
    file_url = urllib.request.pathname2url(file)
    ftype = mimetypes.guess_type(file_url, strict=True)[0]
    ## read data file according to its formate, default includes three types of files: csv/excel/text
    # read csv format data from the parking dataset
    if 'excel' in ftype:
      # usecols: return a subset of the columns, here choose one column to use in the line chart
      data = pd.read_csv(path+file)
    # read excel format data from the parking dataset
    elif 'sheet' in ftype:
      data = pd.read_excel(path+file)
    # read text format data from the parking dataset
    elif ftype == 'text/plain':
      data = pd.read_csv(path+file, sep="\t")
    else:
      print("File type cannot find!")
    return data

  # check the available file name
  # if the input file name already existed then rename to file_1, file_2   
  def get_available_name(self,filename):
    n=[1]
    def check_meta(file_name):
        file_name_new=file_name
        if file_name in [os.path.splitext(i)[0] for i in os.listdir(path_img)]:   
            file_name_new=file_name+'_'+str(n[0])
            n[0]+=1
        if file_name_new in [os.path.splitext(i)[0] for i in os.listdir(path_img)]:   
            file_name_new=check_meta(file_name)
        return file_name_new
    available_name=check_meta(filename)
    return available_name

  # file: file name of your data source
  # x_col_name: ['x_column_name_a','x_column_name_b'...]
  # paper_type : 'single' or 'double'
  def Violin(self,file,x_col_name,paper_type  ,**kwargs):
    # Configuration of the box plot
    # plotwidth: width of the plot
    # plotheight: height of the plot
    # my_font: the typeface of x, y labels
    # backgrid: backgrid of the plot
    # isframe: frame of the plot
    # gridlinewidth: grid line width
    # title: True or False as options. If it is True, add title for the plot
    # title_pad: if the title is True, modify pad size of title
    # title_size: if the title is True, modify size of title
    # title_loc: if the title is True, modify location of title
    # legend_size: size of legend
    # legend_loc: location of legend
    # x_label: the content of x label
    # y_label: the content of y label
    # palette: color palette of box plot
    # width: each violin width
    # vert: True or False, If True , makes the boxes vertical. If False, everything is drawn horizontally
    # showmeans: True or False, if true, show the mean line
    # showextra: True or False, if true, show the extra line
    # showmedian: True or False, if true, show the median line
    # quantiles: show the quantiles line
    # category: category columns of violin plot
    # save_image: True or False as options. If it is True, save chart
    # savefig_bbox_inches: Bounding box in inches
    # file_name: the file name in saving image
    single_column_conf={ 'plotwidth':8,
                  'plotheight':6, 
                  'my_font':'DejaVu Sans',
                  'labeltext_size':25,
                  'labelpad':10,
                  'backgrid':True,
                  'isframe':True,
                  'gridlinewidth':0.5,  
                  'title':False,
                  'title_pad':10,
                  'title_size':20,
                  'tick_size':15,
                  'title_loc':'center',
                  'legend_size': 12,
                  'legend_loc': 'upper right',
                  'x_label':None,
                  'y_label':None,
                  'palette':sns.color_palette("hls", 10),
                  'width':0.5,
                  'vert':True,
                  'showmeans':False,
                  'showextrema':True,
                  'showmedians':True,
                  'quantiles':None,
                  'category':None,
                  'x_scale':1.2,
                  'save_image':False,
                  'savefig_bbox_inches':'tight',
                  'file_name':'violin_plot',
                  }
    double_column_conf={ 'plotwidth':8,
                  'plotheight':6, 
                  'my_font':'DejaVu Sans',
                  'labeltext_size':27,
                  'labelpad':10,
                  'backgrid':True,
                  'isframe':True,
                  'gridlinewidth':0.5,  
                  'title':False,
                  'title_pad':10,
                  'title_size':20,
                  'tick_size':17,
                  'title_loc':'center',
                  'legend_size': 13,
                  'legend_loc': 'upper right',
                  'x_label':None,
                  'y_label':None,
                  'palette':sns.color_palette("hls", 10),
                  'width':0.5,
                  'vert':True,
                  'showmeans':False,
                  'showextrema':True,
                  'showmedians':True,
                  'quantiles':None,
                  'category':None,
                  'x_scale':1.2,
                  'save_image':False,
                  'savefig_bbox_inches':'tight',
                  'file_name':'violin_plot',
                  }
    # choose the configuration of plot
    if paper_type == 'single':
      conf = single_column_conf
    elif paper_type == 'double':
      conf = double_column_conf  
    conf.update(kwargs)

    # create figure and set figure size  
    fig, ax_left = plt.subplots(figsize = (conf['plotwidth'], conf['plotheight']))   

    # read data
    try:
      data = self.read_file(file)
    except Exception:
      print('Sorry, this file does not exist, please check the file name')   
    if conf['backgrid'] == True:
      ax_left.grid(linestyle="--", linewidth=conf['gridlinewidth'], color='gray', alpha=0.5)

    ax_left.set_xlabel(conf['x_label'], fontproperties=conf['my_font'], fontsize=conf['labeltext_size'], labelpad=conf['labelpad'])
    ax_left.set_ylabel(conf['y_label'], fontproperties=conf['my_font'], fontsize=conf['labeltext_size'], labelpad=conf['labelpad'])

   #---plot
    #violinplot with no category
    if conf['category'] == None:
      ticklist=[]
      # arrange all violin plot, each violin use one of palette color
      # the defult violin has medians and extrema line
      for i in range(len(x_col_name)):
        violin=ax_left.violinplot(data[x_col_name[i]],positions=[i],widths=conf['width'],
                          showmeans=conf['showmeans'],showextrema=conf['showextrema'],showmedians=conf['showmedians'],
                          quantiles=conf['quantiles'])
        #set color of each line
        if conf['showmedians'] == True:
          violin['cmedians'].set_color(conf['palette'][i])
        violin['cbars'].set_color(conf['palette'][i])
        if conf['showextrema'] == True:
          violin['cmaxes'].set_color(conf['palette'][i])
          violin['cmins'].set_color(conf['palette'][i])
        if  conf['showmeans'] == True:
          violin['cmeans'].set_color(conf['palette'][i])
        violin['bodies'][0].set_facecolor(conf['palette'][i])
        violin['bodies'][0].set_edgecolor(conf['palette'][i])
        ticklist.append(i)
      # get more blank in x axis
      ax_left_xlim = ax_left.get_xlim()          
      ax_left.set_xlim(ax_left_xlim[0],ax_left_xlim[1]*conf['x_scale']) 
      # change the ticks to columns names
      plt.xticks(ticks=ticklist, labels=x_col_name)
    # category violin plot
    else:  
      #check the category only have 2    
      if len(data[conf['category']].unique())==2:
        cate=data[conf['category']].unique()         
      # separte left and right data  
        dataleft=[]        
        for i in x_col_name:                 
          dataleft.append(np.array(data[data[conf['category']]==cate[0]][i]))
        dataright=[]
        for i in x_col_name:
          dataright.append(np.array(data[data[conf['category']]==cate[1]][i]))
        # get the violin ticks
        ticklist=[]
        for i in range(len(x_col_name)):
          ticklist.append(i)
        # plot the left part of category violin 
        v1 = ax_left.violinplot(dataleft,  positions=np.arange(0, len(dataleft)),
               showmeans=conf['showmeans'], showextrema=conf['showextrema'], showmedians=conf['showmedians'])
        
        for b in v1['bodies']:
        # get the center of left part
          m = np.mean(b.get_paths()[0].vertices[:, 0])
        # modify the paths to not go further right than the center
          b.get_paths()[0].vertices[:, 0] = np.clip(b.get_paths()[0].vertices[:, 0], -np.inf, m)
        # set the color of each infomation line
        if conf['showmedians'] == True:
          v1['cmedians'].set_color(conf['palette'][0])
        v1['cbars'].set_color(conf['palette'][0])
        if conf['showextrema'] == True:
          v1['cmaxes'].set_color(conf['palette'][0])
          v1['cmins'].set_color(conf['palette'][0])
        if  conf['showmeans'] == True:
          v1['cmeans'].set_color(conf['palette'][0])
        # set the facecolor and edgecolor of left part
        for i in range(len(x_col_name)):
          v1['bodies'][i].set_facecolor(conf['palette'][0])
          v1['bodies'][i].set_edgecolor(conf['palette'][0])
        # plot the right part of category violin  
        v2 = ax_left.violinplot(dataright,  positions=np.arange(0, len(dataright)),
               showmeans=conf['showmeans'], showextrema=conf['showextrema'], showmedians=conf['showmedians'])
        for b in v2['bodies']:
          m = np.mean(b.get_paths()[0].vertices[:, 0])
        # modify the paths to not go further left than the center
          b.get_paths()[0].vertices[:, 0] = np.clip(b.get_paths()[0].vertices[:, 0], m, np.inf)
        if conf['showmedians'] == True:
          v2['cmedians'].set_color(conf['palette'][5])
        v2['cbars'].set_color(conf['palette'][5])
        if conf['showextrema'] == True:
          v2['cmaxes'].set_color(conf['palette'][5])
          v2['cmins'].set_color(conf['palette'][5])
        if  conf['showmeans'] == True:
          v2['cmeans'].set_color(conf['palette'][5])
        for i in range(len(x_col_name)):
          v2['bodies'][i].set_facecolor(conf['palette'][5])
          v2['bodies'][i].set_edgecolor(conf['palette'][5])
        # get more blank in x axis
        ax_left_xlim = ax_left.get_xlim()          
        ax_left.set_xlim(ax_left_xlim[0],ax_left_xlim[1]*conf['x_scale']) 
        ax_left.legend([v1['bodies'][0],v2['bodies'][0]],[cate[0], cate[1]],loc=conf['legend_loc'], fontsize=conf['legend_size'])
        # change the ticks to columns names
        plt.xticks(ticks=ticklist, labels=x_col_name)
      # if the category number !=2 report mistake
      else:
        print('Sorry, the category number can only be two') 

    # if False, top and right borders removing    
    if conf['isframe'] == False:
        ax.spines['top'].set_visible(False)
        ax.spines['right'].set_visible(False)

    # set x, y tick's direction, default:out, can be set to in,out,inout
    for tick in ax_left.xaxis.get_major_ticks():
      tick.label.set_fontsize(conf['tick_size'])
    for tick in ax_left.yaxis.get_major_ticks():
      tick.label.set_fontsize(conf['tick_size'])
    # title setting
    if conf['title'] == False:
      pass
    else:
      ax_left.set_title(conf['title'], fontsize=conf['title_size'], loc=conf['title_loc'], pad=conf['title_pad'])
    #save plot setting
    if conf['save_image'] == True:
      file_name=conf['file_name']
      file_newname = self.get_available_name(file_name)
      plt.savefig(path_img+file_newname, bbox_inches=conf['savefig_bbox_inches'],dpi=600,format='jpg')  
    plt.show()     


import mimetypes
import numpy as np
import pandas as pd
import seaborn as sns
import matplotlib as mpl
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
from mpl_toolkits.axes_grid1.inset_locator import zoomed_inset_axes 
from mpl_toolkits.axes_grid1.inset_locator import mark_inset

class TimeSeries:
    """
    A time-series data visualization class created based on matplotlib. This class provides
    single plot or multiple plots of dataset variables over a time interval. Currenly supports
    marking and annotating minimum/maximum points, and annual peak/trough points, as well as
    zooming-in a specified region of a plot.
    
    Usage:
        ts = TimeSeries(file)
        ts.plot('x', ['y1', 'y2'], start_time='2000', end_time='2010')
    """
    
    def __init__(self, file):
        try:
            self.data = self.read_file(file)
        except FileNotFoundError:
            print('This file does not exist, please check the file name.')
        else:
            # keyword arguments
            self.conf = {
                'title': None, 'title_size': 16,
                'xlabel': None, 'ylabel': [],
                'xlabel_size': 15, 'ylabel_size': 15,
                'xtick_rotate': 0, 'xtick_size': 10,
                'ytick_rotate': 0, 'ytick_size': 10,
                'ylim_bot': None, 'ylim_top': None,
                'my_font': 'DejaVu Sans',
                'legend': [],
                'color': [],
                'backgrid': True,
                'grid_width': 0.5,
                'grid_alpha': 0.5,
                'figsize': (6, 4),
                'show_min': False,
                'show_max': False,
                'min_annotate': False,
                'max_annotate': False,
                'precision': 1,
                'min_size': 15, 'max_size': 15,
                'min_marker': 'v', 'max_marker': '^',
                'min_color': 'red', 'max_color': 'green',
                'show_peaks': False, 
                'show_troughs': False,
                'peak_annotate': False,
                'trough_annotate': False,
                'peak_size': 10, 'trough_size': 10,
                'peak_color': 'green',
                'trough_color': 'red',
                'peak_label': 'Annual Peaks',
                'trough_label': 'Annual Troughs',
                'peak_marker': '^',
                'trough_marker': 'v',
                'zoom_in': False,
                'x_start': [], 'x_end': [],
                'y_start': [], 'y_end': [],
                'zoom_scale': 3,
                'zoom_x_rotate': 90,
                'zoom_loc': 'upper right',
                'multiple_plot': False,
                'date_fmt': None,
                'save_fig': False,
                'fig_path': None,
            }
    
    def read_file(self, file):
        """Read csv, excel, text format file
        Args:
            file: the input dataset
            
        Returns:
            The data read from the file  
        """
        ftype = mimetypes.guess_type(file, strict=True)[0]
        if ftype is None:
            print('Cannot guess file type!')
            return None
        # read csv format data
        if 'excel' in ftype:
            data = pd.read_csv(file)
        # read excel format data
        elif 'sheet' in ftype:
            data = pd.read_excel(file)
        # read text format data
        elif ftype == 'text/plain':
            data = pd.read_csv(file)
        else:
            data = None
            print("File type cannot find!")
        return data
    
    def plot(self, x_col_name, y_col_name, start_time=None, end_time=None, **kwargs):
        """show one or more variables over a period of time
        Args:
            x_col_name: name of the date variable
            y_col_name: list of names for y variable
            start_time: the start date on the timeline
            end_time: the end date on the timeline
            kwargs: a dictionary of optional parameters
            
        Returns:
            None
        """
        
        # when new self.configuraton is set, update the original one
        self.conf.update(kwargs)

        # convert dates to pandas datetime format
        self.data[x_col_name] = pd.to_datetime(self.data[x_col_name])
        
        # extract data from selected time interval
        start_t = (pd.to_datetime(start_time) if start_time
                   else self.data[x_col_name].min())
        end_t = (pd.to_datetime(end_time) if end_time
                else self.data[x_col_name].max())
        data = self.data.loc[(self.data[x_col_name] >= start_t) &
                             (self.data[x_col_name] <= end_t)] 
        data = data.set_index(x_col_name)
        
        # initialize color
        if len(self.conf['color']) == 0:
            self.conf['color'] = sns.color_palette('muted')
        
        if self.conf['multiple_plot']:
            fig, axs = plt.subplots(len(y_col_name), 1, sharex=True,
                            figsize=self.conf['figsize'])
        else:
            fig, ax = plt.subplots(1, 1, figsize=self.conf['figsize'])
            axs = [ax for i in y_col_name]
        
        # plot xy points
        x = data.index
        for i, y_col in enumerate(y_col_name):
            y = data[y_col].values
            color = self.conf['color'][i] if i <= len(self.conf['color'])-1 else 'red'
            label = self.conf['legend'][i] if i <= len(self.conf['legend'])-1 else None
            axs[i].plot(x, y, color=color, label=label)
            if label is not None:
                axs[i].legend()

        # background grid setting
        if self.conf['backgrid']:
            for i, _ in enumerate(y_col_name):
                axs[i].grid(linestyle="--", linewidth=self.conf['grid_width'],
                        color='gray', alpha=self.conf['grid_alpha'])
            
        # set title and size
        axs[0].set_title(self.conf['title'], fontproperties=self.conf['my_font'],
                         fontsize=self.conf['title_size'])

        # set xy labels and size
        axs[-1].set_xlabel(self.conf['xlabel'], fontproperties=self.conf['my_font'],
                           fontsize=self.conf['xlabel_size'])
        for i, _ in enumerate(y_col_name):
            ylabel = self.conf['ylabel'][i] if i <= len(self.conf['ylabel'])-1 else None
            axs[i].set_ylabel(ylabel, fontproperties=self.conf['my_font'],
                              fontsize=self.conf['ylabel_size'])
            if not self.conf['multiple_plot']: break

        # set xy limits (only allowed for single plot)
        if not self.conf['multiple_plot']:
            hi = [max(data[y_col_name[j]]) for j in range(len(y_col_name))]
            lo = [min(data[y_col_name[j]]) for j in range(len(y_col_name))]
            ylim_bot = (min(lo)-max(hi)*0.1 if self.conf['ylim_bot'] is None
                        else self.conf['ylim_bot'])
            ylim_top = (max(hi)*1.1 if self.conf['ylim_top'] is None
                        else self.conf['ylim_top'])
            axs[0].set_ylim(bottom=ylim_bot, top=ylim_top)
        
        # tick labels and size, rotatation
        plt.xticks(ticks=data.index, labels=data.index,
                rotation=self.conf['xtick_rotate'], fontsize=self.conf['xtick_size'])
        plt.yticks(rotation=self.conf['ytick_rotate'], fontsize=self.conf['ytick_size'])
        
        # show dates in "year-month-day" format on x axis
        xlocator = mdates.AutoDateLocator()
        xformatter = mdates.AutoDateFormatter(xlocator)
        axs[0].xaxis.set_major_locator(xlocator)
        axs[0].xaxis.set_major_formatter(xformatter)
        
        if self.conf['show_max']:
            offset = max(max(data[c]) for c in y_col_name)*0.02
            for i, y_col in enumerate(y_col_name):
                y = data[y_col].values
                argmax = np.argmax(y)
                # plot max point
                axs[i].scatter(x[argmax], y[argmax],
                           marker=self.conf['max_marker'],
                           color=self.conf['max_color'])
                if self.conf['max_annotate']:
                    # mark max point
                    y_offset = max(y)*0.02 if self.conf['multiple_plot'] else offset
                    tx = (x[argmax].strftime(self.conf['date_fmt'])
                          if self.conf['date_fmt']
                          else x[argmax].date())
                    ty = y[argmax]
                    show_max = ('[{}, {:.{}e}]'.format(tx, ty, self.conf['precision'])
                               if len(str(ty).split(',')) > 5
                               else '[{}, {:.{}f}]'.format(tx, ty, self.conf['precision']))
                    axs[i].annotate(show_max, xy=(x[argmax],y[argmax]),
                             xytext=(x[argmax], y[argmax]+y_offset),
                             ha="center", va="bottom")
        
        if self.conf['show_min']:
            offset = max(max(data[c]) for c in y_col_name)*0.02
            for i, y_col in enumerate(y_col_name):
                y = data[y_col].values
                argmin = np.argmin(y)
                # plot min point
                axs[i].scatter(x[argmin], y[argmin],
                           marker=self.conf['min_marker'],
                           color=self.conf['min_color'])
                if self.conf['min_annotate']:
                    # mark min point
                    y_offset = max(y)*0.02 if self.conf['multiple_plot'] else offset
                    tx = (x[argmin].strftime(self.conf['date_fmt'])
                          if self.conf['date_fmt']
                          else x[argmin].date())
                    ty = y[argmin]
                    show_min = ('[{}, {:.{}e}]'.format(tx, ty, self.conf['precision'])
                               if len(str(ty).split(',')) > 5
                               else '[{}, {:.{}f}]'.format(tx, ty, self.conf['precision']))
                    axs[i].annotate(show_min, xy=(x[argmin],y[argmin]),
                             xytext=(x[argmin], y[argmin]-y_offset),
                             ha="center", va="top")
        
        if self.conf['show_peaks']:
            for i, y_col in enumerate(y_col_name):
                # identify location of peaks
                y = data[y_col].values
                years = np.unique([str(d.year) for d in data.index]).tolist()
                x_peaks = [data.loc[y:y, y_col].idxmax() for y in years[:-1]]
                y_peaks = data.loc[x_peaks, y_col]
                # plot peak points and legend
                axs[i].scatter(x_peaks, y_peaks, 
                        marker=self.conf['peak_marker'],
                        color=self.conf['peak_color'],
                        label=self.conf['peak_label'] if i==0 or i>0 and self.conf['multiple_plot'] else '')
                axs[i].legend()
                if self.conf['peak_annotate']:
                    # annotate location of peaks
                    for xp, yp in zip(x_peaks, y_peaks):
                        axs[i].text(xp, yp+max(y_peaks)*0.01,
                                fontsize=self.conf['peak_size'],
                                ha='center', va='bottom',
                                color=self.conf['peak_color'],
                                s=(xp.strftime(self.conf['date_fmt'])
                                   if self.conf['date_fmt'] else xp.date()))
        
        if self.conf['show_troughs']:
            for i, y_col in enumerate(y_col_name):
                # identify location of troughs
                y = data[y_col].values
                years = np.unique([str(d.year) for d in data.index]).tolist()
                x_troughs = [data.loc[y:y, y_col].idxmin() for y in years[:-1]]
                y_troughs = data.loc[x_troughs, y_col]
                # plot trough points and legend
                axs[i].scatter(x_troughs, y_troughs,
                        marker=self.conf['trough_marker'],
                        color=self.conf['trough_color'],
                        label=self.conf['trough_label'] if i==0 or i>0 and self.conf['multiple_plot'] else '')
                axs[i].legend()
                if self.conf['trough_annotate']:
                     # annotate location of troughs
                    for xt, yt in zip(x_troughs, y_troughs):
                        axs[i].text(xt, yt-max(y_troughs)*0.01,
                                fontsize = self.conf['trough_size'],
                                ha='center', va='top',
                                color=self.conf['trough_color'],
                                s=(xt.strftime(self.conf['date_fmt'])
                                   if self.conf['date_fmt'] else xt.date()))
        
        if self.conf['zoom_in']:
            for i, y_col in enumerate(y_col_name):
                x_start = pd.to_datetime(self.conf['x_start'][i])
                x_end = pd.to_datetime(self.conf['x_end'][i])
                y_start = self.conf['y_start'][i]
                y_end = self.conf['y_end'][i]

                axins = zoomed_inset_axes(ax, self.conf['zoom_scale'], loc=self.conf['zoom_loc'])
                axins.plot(x, data[y_col], color='#0dd0e0')
                axins.set_xlim(x_start, x_end)
                axins.set_ylim(y_start, y_end)
                axins.tick_params(axis='x', rotation=self.conf['zoom_x_rotate'])
                
                zlocator = mdates.AutoDateLocator()
                zformatter = mdates.AutoDateFormatter(zlocator)
                axins.xaxis.set_major_locator(zlocator)
                axins.xaxis.set_major_formatter(zformatter)
                mark_inset(axs[i], axins, loc1=2, loc2=4, fc="0.7", ec="0.7")
        
        if self.conf['save_fig']:
            fig_path = ("ts_plot.png" if self.conf['fig_path'] is None
                        else self.conf['fig_path'])
            plt.savefig(fig_path, dpi=500)
        plt.show()


# Data analysis library numpy and pandas
import pandas as pd
import numpy as np

# Data visualization library matplotlib and seaborn
import seaborn as sns
import matplotlib
import matplotlib.pyplot as plt
import matplotlib.font_manager
import matplotlib.patches as mpatches

import mimetypes
import urllib
import os


#from google.colab import drive	
#drive.mount('/content/drive')

## setting path
# get current path
path_current = os.getcwd()

path_current=path_current.replace('\\', '/')
# the path is where the dataset saved
path = path_current + '/Data/Pie/' 
# the "path_img" is the position where final image will be saved
path_img = path_current + '/Images/'

class Pie_plot: 
  # read data
  def read_file(self,file):
    file_url = urllib.request.pathname2url(file)
    ftype = mimetypes.guess_type(file_url, strict=True)[0]
    ## read data file according to its formate, default includes three types of files: csv/excel/text
    # read csv format data from the parking dataset
    if 'excel' in ftype:
      # usecols: return a subset of the columns, here choose one column to use in the line chart
      data = pd.read_csv(path+file)
    # read excel format data from the parking dataset
    elif 'sheet' in ftype:
      data = pd.read_excel(path+file)
    # read text format data from the parking dataset
    elif ftype == 'text/plain':
      data = pd.read_csv(path+file, sep="\t")
    else:
      print("File type cannot find!")
    return data

  def explode(self, label, explode_label,explode_value):
    explode_list=[explode_value if i in explode_label else 0 for i in label]
    return explode_list
  
  def get_available_name(self,filename):
    n=[1]
    def check_meta(file_name):
        file_name_new=file_name
        if file_name in [os.path.splitext(i)[0] for i in os.listdir(path_img)]:   
            file_name_new=file_name+'_'+str(n[0])
            n[0]+=1
        if file_name_new in [os.path.splitext(i)[0] for i in os.listdir(path_img)]:   
            file_name_new=check_meta(file_name)
        return file_name_new
    available_name=check_meta(filename)
    return available_name
  
  # file: file name of your data source
  # value: the columns name of pie chart value
  # label: the columns name of pie chart label
  # paper_type: 'single' or 'double'

  def Pie(self, file, value, label, paper_type, **kwargs):
    # Configuration of the pie chart
    # plotwidth: width of the plot
    # plotheight: height of the plot
    # my_font: the typeface of x, y labels
    # labeltext_size: the size of label
    # legend_size: size of legend
    # legend_loc: location of legend of labels
    # layer_legend_loc: location of legend of layers
    # linewidth: the width of each wedge outline
    # edgecolor: the color of wedge outline
    # title: True or False as options. If it is True, add title for the plot
    # title_pad: if the title is True, modify pad size of title
    # title_size: if the title is True, modify size of title
    # title_loc: if the title is True, modify location of title
    # radius: the radius of pie chart
    # explode: True or false, if true, some wedge will explore outside, only the single layer can explode
    # explode_value: when explode is True, the distance of stick out
    # explode_label: when explode is True, the label of stick out
    # autopct: the format of value
    # actual_values: True or False, if true, show the actual value of each label
    # pctdistance: the distance of value
    # labeldistance: the distance of label
    # info_style: when there is only one value input, the annotate style: simple, change_font, change_radius, change_both
                  # simple: the basic pie chart
                  # change_font: the font size will change by area(value), the more value means bigger font size
                  # change_radius: the radius will change by area(value), the more value means bigger radius
                  # change_both: the font size and radiuswill change by area(value), the more value means bigger font size and radius
    # radius_scale: the scale of radius when info_style has some change in radius
    # fontsize_scale: the scale of font size when info_style has some change in font size
    # label_pos_scale: when the radius change, the label position scale
    # val_pos_scale: when the radius change, the value position scale
    # startangle: the pie chart start angle
    # font_size: the font size of value
    # palette: the color palette of pie chart
    # shadow: True or False, the shadow of pie chart
    # layer_scale: if there are multiple input value, the multi-layer radius scale
    # donut: True or False, if ture, draw donut plot
    # donut_circle: if donut is True, the position and radius of donut white blank
    # save_image: True or False as options. If it is True, save chart
    # savefig_bbox_inches: Bounding box in inches
    # file_name: the file name in saving image
    single_column_conf={ 'plotwidth':8,
                  'plotheight':8, 
                  'my_font':'DejaVu Sans',
                  'labeltext_size':17,
                  'legend_size':8,
                  'legend_loc':'upper right', 
                  'layer_legend_loc' :'upper left',
                  'linewidth' : 1.5,
                  'edgecolor' : 'white',
                  'title':False,
                  'title_pad':10,
                  'title_size':20,
                  'title_loc':'center', 
                  'radius':1, 
                  'explode':False, 
                  'explode_value':0.1, 
                  'explode_label':None, 
                  'autopct':'%.2f%%',
                  'actual_values':False, 
                  'pctdistance':0.75, 
                  'labeldistance':1.05,
                  'info_style':'Simple',
                  'radius_scale':1.02,  
                  'fontsize_scale':1.05,
                  'label_pos_scale':1.1,
                  'val_pos_scale':0.6,
                  'startangle':90,
                  'font_size':11,
                  'palette':sns.color_palette("hls", 10),
                  'shadow':False,
                  'layer_scale':0.65,
                  'donut':False,
                  'donut_circle': [(0,0), 0.3,'white'],
                  'save_image':False,
                  'savefig_bbox_inches':'tight',
                  'file_name':'pie_chart'            
                  } 

    double_column_conf={ 'plotwidth':8,
                  'plotheight':8, 
                  'my_font':'DejaVu Sans',
                  'labeltext_size':17,
                  'legend_size':9,
                  'legend_loc':'upper right',
                  'layer_legend_loc' :'upper left',
                  'linewidth' : 1.5,
                  'edgecolor' : 'white',
                  'title':False,
                  'title_pad':10,
                  'title_size':20,
                  'title_loc':'center', 
                  'radius':1, 
                  'explode':False, 
                  'explode_value':0.1, 
                  'explode_label':None, 
                  'autopct':'%.2f%%',
                  'actual_values':False, 
                  'pctdistance':0.75, 
                  'labeldistance':1.05,
                  'info_style':'Simple',
                  'radius_scale':1.02,  
                  'fontsize_scale':1.05,
                  'label_pos_scale':1.1,
                  'val_pos_scale':0.6,
                  'startangle':90,
                  'font_size':12,
                  'palette':sns.color_palette("hls", 10),
                  'shadow':False,
                  'layer_scale':0.65,
                  'donut':False,
                  'donut_circle': [(0,0), 0.3,'white'],
                  'save_image':False,
                  'savefig_bbox_inches':'tight',
                  'file_name':'pie_chart'                                   
                  } 

    if paper_type == 'single':
      conf = single_column_conf
    elif paper_type == 'double':
      conf = double_column_conf  
    conf.update(kwargs)

  # create figure and set figure size  
    fig, ax_left = plt.subplots(figsize = (conf['plotwidth'], conf['plotheight']))   

  #read data
    try:
      data = self.read_file(file)
    except Exception:
      print('Sorry, this file does not exist, please check the file name') 

    # get actual value
    def make_autopct(values):
      def my_autopct(pct):
        total = sum(values)
        val = int(round(pct*total/100.0))
        return '{p:.2f}% \n {v:d}'.format(p=pct,v=val)
      return my_autopct
    
    # Plot  
    # no explode
    if conf['explode'] == False:
    
    # single layer
      if len(value) == 1: 
        #simple info style
        if conf['info_style'] =='Simple':
        #show actual value
          if conf['actual_values'] == True:
            autopct=make_autopct(data[value[0]])
          # dont show actual value
          else:
            autopct=conf['autopct']

          wedges, texts, autotexts=ax_left.pie(x=data[value[0]],labels=data[label[0]],autopct=autopct,pctdistance=conf['pctdistance'], labeldistance=conf['labeldistance'],
                    wedgeprops={ 'linewidth' : conf['linewidth'], 'edgecolor' : conf['edgecolor'] },colors=conf['palette'],startangle=conf['startangle'],textprops={'fontsize':conf['labeltext_size']},radius=conf['radius']) 
          #rotate the value text
          for i, p in enumerate(wedges):
            ang = (p.theta2 - p.theta1)/2. + p.theta1
            if ang >=90 and ang <= 270:
              plt.setp(autotexts[i],rotation=ang-180, ha="center",va='center',rotation_mode="anchor")
            else:  
              plt.setp(autotexts[i],rotation=ang, ha="center",va='center',rotation_mode="anchor")        
        # DIY info style(outside_box,change_radius,change_font,change_both)
        else:
          #show actual value
          if conf['actual_values'] == True:
            autopct=make_autopct(data[value[0]])
          # dont show
          else:
            autopct=conf['autopct']
          # outside box
          if conf['info_style']=='Outside_box':            
            #get each part's wedges, texts, autotexts
            wedges, texts, autotexts=ax_left.pie(x=data[value[0]],labels=data[label[0]],autopct=autopct,
                        wedgeprops={ 'linewidth' : conf['linewidth'], 'edgecolor' : conf['edgecolor'] },textprops={'alpha':0},colors=conf['palette'],startangle=conf['startangle'],radius=conf['radius']) 
            # draw info box
            bbox_props = dict(boxstyle="square,pad=0.3", fc="w", ec="k", lw=0.5,alpha=0)
            kw = dict(arrowprops=dict(arrowstyle="-"),bbox=bbox_props, zorder=0, va="center",size=conf['font_size'])
            
            # create a list to store label and value
            infolist=[]
            for i in range(len(texts)):
              infolist.append(str(texts[i].get_text())+' '+str(autotexts[i].get_text()))
            # get the info position
            for i, p in enumerate(wedges):
              ang = (p.theta2 - p.theta1)/2. + p.theta1
              y = np.sin(np.deg2rad(ang))
              x = np.cos(np.deg2rad(ang))
              horizontalalignment = {-1: "right", 1: "left"}[int(np.sign(x))]
              connectionstyle = "angle,angleA=0,angleB={}".format(ang)
              kw["arrowprops"].update({"connectionstyle": connectionstyle})
              ax_left.annotate(infolist[i], xy=(x, y), xytext=(1.1*np.sign(x), 1.1*y),
                          horizontalalignment=horizontalalignment, **kw)
          #(change_radius,change_font,change_both)    
          else:  
            wedges, texts, autotexts=ax_left.pie(x=data[value[0]],labels=data[label[0]],autopct=autopct,pctdistance=conf['pctdistance'], labeldistance=conf['labeldistance'],
                        wedgeprops={ 'linewidth' : conf['linewidth'], 'edgecolor' : conf['edgecolor'] },colors=conf['palette'],startangle=conf['startangle'],radius=conf['radius']) 
            # create a list to store each part angle
            anglist=[]
            for i,p in enumerate(wedges):
              angrange = (p.theta2 - p.theta1)
              anglist.append((i,angrange))
              #rotate the value text
              ang = (p.theta2 - p.theta1)/2. + p.theta1
              if ang >=90 and ang <= 270:
                plt.setp(autotexts[i],rotation=ang-180, ha="center",va='center',rotation_mode="anchor")
              else:  
                plt.setp(autotexts[i],rotation=ang, ha="center",va='center',rotation_mode="anchor") 
             # create a list to store each part label 
            labellist=[]
            for i,p in enumerate(texts):
              labellist.append(p.get_text())
            all=list(zip(labellist,anglist))

            # define the sort key value
            def findkey(elem):
              return elem[1][1]
            # sort parts by  area   
            all.sort(key=findkey)
            #get the sorted list
            sortedlist=[]
            for i in all:
              sortedlist.append(i[1][0])

            radius=conf['radius']
            fontsize=conf['font_size']
            # the smallest one has smallest font size and radius, then scale font and radius in other parts
            for i in sortedlist:
              if conf['info_style']=='Change_radius':
                # set the different wedge radius
                wedges[i].set_radius(radius)
                radius=radius*conf['radius_scale']
                #get the middle angle of each wedge
                ang=(wedges[i].theta2-wedges[i].theta1)/2+wedges[i].theta1
                y = np.sin(np.deg2rad(ang))*wedges[i].r
                x = np.cos(np.deg2rad(ang))*wedges[i].r
                # put the label and value in new position
                plt.setp(autotexts[i],x=x*conf['val_pos_scale'],y=y*conf['val_pos_scale'] ,size=conf['font_size'])
                plt.setp(texts[i],x=x*conf['label_pos_scale'],y=y*conf['label_pos_scale'] ,size=conf['font_size'])
              if conf['info_style']=='Change_font':
                #get the middle angle of each wedge
                ang=(wedges[i].theta2-wedges[i].theta1)/2+wedges[i].theta1
                y = np.sin(np.deg2rad(ang))*wedges[i].r
                x = np.cos(np.deg2rad(ang))*wedges[i].r
                # put the label and value in new font size
                plt.setp(autotexts[i],x=x*conf['val_pos_scale'],y=y*conf['val_pos_scale'] ,size=fontsize)
                plt.setp(texts[i],x=x*conf['label_pos_scale'],y=y*conf['label_pos_scale'] ,size=fontsize)
                #change the font size by scale
                fontsize=fontsize*conf['fontsize_scale']
              if conf['info_style']=='Change_both':
                wedges[i].set_radius(radius)
                radius=radius*conf['radius_scale']
                ang=(wedges[i].theta2-wedges[i].theta1)/2+wedges[i].theta1
                y = np.sin(np.deg2rad(ang))*wedges[i].r
                x = np.cos(np.deg2rad(ang))*wedges[i].r
                plt.setp(autotexts[i],x=x*conf['val_pos_scale'],y=y*conf['val_pos_scale'] ,size=fontsize)
                plt.setp(texts[i],x=x*conf['label_pos_scale'],y=y*conf['label_pos_scale'] ,size=fontsize)
                fontsize=fontsize*conf['fontsize_scale']
        # if true, add a write circle in the middle, the plot changes to donut chart
        if conf['donut'] ==True:
          circle=plt.Circle( conf['donut_circle'][0],conf['donut_circle'][1], color='white')
          p=plt.gcf()
          p.gca().add_artist(circle)      
      
      
      #multi layer  (if the length of input value !=1 ) 
      else:
        radius=conf['radius']
        pctdistance=conf['pctdistance']
        labeldistance=conf['labeldistance']
        for i in range(len(value)):
          if conf['actual_values'] == True:
            autopct=make_autopct(data[value[-(i+1)]])
          else:
            autopct=conf['autopct']

          if i == 0:
            #draw the first layer(most outside layer)
            wedges, texts, autotexts=ax_left.pie(x=data[value[-(i+1)]],labels=data[label[0]],autopct=autopct,pctdistance=pctdistance, labeldistance=labeldistance,
                        wedgeprops={ 'linewidth' : conf['linewidth'], 'edgecolor' : conf['edgecolor'] },textprops={'fontsize':conf['font_size']},colors=conf['palette'],startangle=conf['startangle'],radius=radius) 
            # hide outside label show
            plt.setp(texts,alpha=0)
            for i, p in enumerate(wedges):
              ang = (p.theta2 - p.theta1)/2. + p.theta1
              if ang >=90 and ang <= 270:
                plt.setp(autotexts[i],rotation=ang-180, ha="center",va='center',rotation_mode="anchor")
              else:  
                plt.setp(autotexts[i],rotation=ang, ha="center",va='center',rotation_mode="anchor")    

          else:
            # draw the inside layers
            wedges, texts, autotexts=ax_left.pie(x=data[value[-(i+1)]],autopct=autopct,pctdistance=pctdistance,
                        wedgeprops={ 'linewidth' : conf['linewidth'], 'edgecolor' : conf['edgecolor'] },colors=conf['palette'],textprops={'fontsize':conf['font_size']},startangle=conf['startangle'],radius=radius) 
            for i, p in enumerate(wedges):
              ang = (p.theta2 - p.theta1)/2. + p.theta1
              if ang >=90 and ang <= 270:
                plt.setp(autotexts[i],rotation=ang-180, ha="center",va='center',rotation_mode="anchor")
              else:  
                plt.setp(autotexts[i],rotation=ang, ha="center",va='center',rotation_mode="anchor")    

          #get the new radius of inside layers
          radius =radius*conf['layer_scale']
          
        # get the layer name
        legend_list=[]          
        for i in range(len(value)):           
          legend=mpatches.Circle((0,0), 0,alpha=0,label='layer'+str((i+1))+'-'+value[i])
          legend_list.append(legend) 
        # get the legend of layers         
        layer_legend=plt.legend(handles=legend_list,loc=conf['layer_legend_loc'])
        # get the lengend of labels
        plt.legend(ncol=2,loc=conf['legend_loc'])
        plt.gca().add_artist(layer_legend)

        if conf['donut'] == True:
          circle_pos=conf['donut_circle'][0]
          circle_rad=radius*conf['layer_scale']
          circle=plt.Circle( conf['donut_circle'][0],circle_rad, color=conf['donut_circle'][2])
          p=plt.gcf()
          p.gca().add_artist(circle)  

    # have explode
    if conf['explode'] == True:    
      if conf['info_style'] =='Simple':
        if conf['actual_values'] == True:
          autopct=make_autopct(data[value[0]])
        else:
          autopct=conf['autopct'] 
          # get explode list   
        explode_list=self.explode(data[label[0]], conf['explode_label'],conf['explode_value'])

        wedges, texts, autotexts = ax_left.pie(x=data[value[0]],labels=data[label[0]],autopct=autopct,pctdistance=conf['pctdistance'], labeldistance=conf['labeldistance'],
                    wedgeprops={'linewidth' : conf['linewidth'], 'edgecolor' : conf['edgecolor'] },textprops={'fontsize':conf['labeltext_size']},colors=conf['palette']
                    ,startangle=conf['startangle'],radius=conf['radius'],explode=explode_list)
        
        for i, p in enumerate(wedges):
          ang = (p.theta2 - p.theta1)/2. + p.theta1
          if ang >=90 and ang <= 270:
            plt.setp(autotexts[i],rotation=ang-180, ha="center",va='center',rotation_mode="anchor")
          else:  
            plt.setp(autotexts[i],rotation=ang, ha="center",va='center',rotation_mode="anchor")    
      else:
        #have actual value
        if conf['actual_values'] == True:
          autopct=make_autopct(data[value[0]])
        # dont have
        else:
          autopct=conf['autopct']
        explode_list=self.explode(data[label[0]], conf['explode_label'],conf['explode_value'])

        if conf['info_style']=='Outside_box':
            wedges, texts, autotexts=ax_left.pie(x=data[value[0]],labels=data[label[0]],autopct=autopct,
                        wedgeprops={ 'linewidth' : conf['linewidth'], 'edgecolor' : conf['edgecolor'] },textprops={'alpha':0},colors=conf['palette'],startangle=conf['startangle'],radius=conf['radius'],explode=explode_list) 
            bbox_props = dict(boxstyle="square,pad=0.3", fc="w", ec="k", lw=0.72,alpha=0)
            kw = dict(arrowprops=dict(arrowstyle="-"),bbox=bbox_props, zorder=0, va="center",size=conf['font_size'])
            infolist=[]
            for i in range(len(texts)):
              infolist.append(str(texts[i].get_text())+' '+str(autotexts[i].get_text()))
            for i, p in enumerate(wedges):
              ang = (p.theta2 - p.theta1)/2. + p.theta1
              y = np.sin(np.deg2rad(ang))
              x = np.cos(np.deg2rad(ang))
              horizontalalignment = {-1: "right", 1: "left"}[int(np.sign(x))]
              connectionstyle = "angle,angleA=0,angleB={}".format(ang)
              kw["arrowprops"].update({"connectionstyle": connectionstyle})
              ax_left.annotate(infolist[i], xy=(x, y), xytext=(1.1*np.sign(x), 1.1*y),
                          horizontalalignment=horizontalalignment, **kw)         
        else:
          wedges, texts, autotexts=ax_left.pie(x=data[value[0]],labels=data[label[0]],autopct=autopct,pctdistance=conf['pctdistance'], labeldistance=conf['labeldistance'],
                      wedgeprops={ 'linewidth' : conf['linewidth'], 'edgecolor' : conf['edgecolor'] },colors=conf['palette'],startangle=conf['startangle'],radius=conf['radius'],explode=explode_list) 
          anglist=[]
          for i,p in enumerate(wedges):
            angrange = (p.theta2 - p.theta1)
            anglist.append((i,angrange))
            ang = (p.theta2 - p.theta1)/2. + p.theta1
            if ang >=90 and ang <= 270:
              plt.setp(autotexts[i],rotation=ang-180, ha="center",va='center',rotation_mode="anchor")
            else:  
              plt.setp(autotexts[i],rotation=ang, ha="center",va='center',rotation_mode="anchor")    
          labellist=[]
          for i,p in enumerate(texts):
            labellist.append(p.get_text())
          all=list(zip(labellist,anglist))
          def findkey(elem):
            return elem[1][1]
          all.sort(key=findkey)
          sortedlist=[]
          for i in all:
            sortedlist.append(i[1][0])
          radius=conf['radius']
          fontsize=conf['font_size']
          labelsize=conf['labeltext_size']
          for i in sortedlist:
            if conf['info_style']=='Change_radius':
              wedges[i].set_radius(radius)
              radius=radius*conf['radius_scale']
              ang=(wedges[i].theta2-wedges[i].theta1)/2+wedges[i].theta1
              y = np.sin(np.deg2rad(ang))*wedges[i].r
              x = np.cos(np.deg2rad(ang))*wedges[i].r
              plt.setp(autotexts[i],x=x*conf['val_pos_scale'],y=y*conf['val_pos_scale'] ,size=conf['font_size'])
              plt.setp(texts[i],x=x*conf['label_pos_scale'],y=y*conf['label_pos_scale'] ,size=conf['labeltext_size'])
            if conf['info_style']=='Change_font':
              ang=(wedges[i].theta2-wedges[i].theta1)/2+wedges[i].theta1
              y = np.sin(np.deg2rad(ang))*wedges[i].r
              x = np.cos(np.deg2rad(ang))*wedges[i].r
              plt.setp(autotexts[i],x=x*conf['val_pos_scale'],y=y*conf['val_pos_scale'] ,size=fontsize)
              plt.setp(texts[i],x=x*conf['label_pos_scale'],y=y*conf['label_pos_scale'] ,size=labelsize)
              fontsize=fontsize*conf['fontsize_scale']
              labelsize=labelsize*conf['fontsize_scale']
            if conf['info_style']=='Change_both':
              wedges[i].set_radius(radius)
              radius=radius*conf['radius_scale']
              ang=(wedges[i].theta2-wedges[i].theta1)/2+wedges[i].theta1
              y = np.sin(np.deg2rad(ang))*wedges[i].r
              x = np.cos(np.deg2rad(ang))*wedges[i].r
              plt.setp(autotexts[i],x=x*conf['val_pos_scale'],y=y*conf['val_pos_scale'] ,size=fontsize)
              plt.setp(texts[i],x=x*conf['label_pos_scale'],y=y*conf['label_pos_scale'] ,size=fontsize)
              fontsize=fontsize*conf['fontsize_scale']
              labelsize=labelsize*conf['fontsize_scale']  

      if conf['donut'] == True:
        circle=plt.Circle( conf['donut_circle'][0],conf['donut_circle'][1], color=conf['donut_circle'][2])
        p=plt.gcf()
        p.gca().add_artist(circle) 
    
    if conf['title'] == False:
      pass
    else:
      ax_left.set_title(conf['title'], fontsize=conf['title_size'], loc=conf['title_loc'], pad=conf['title_pad'])

    #save
    if conf['save_image'] == True:
      file_name=conf['file_name']
      file_newname = self.get_available_name(file_name)
      plt.savefig(path_img+file_newname, bbox_inches=conf['savefig_bbox_inches'],dpi=600,format='jpg') 
     
    # showing the image
    plt.show()  
      


import os
from setuptools import setup, find_packages

# Get description from README
root = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(root, 'README.md'), 'r', encoding='utf-8') as f:
    long_description = f.read()


setup(
    name='paperviz',
    version='1.0.7',
    description='Professional Python Graphs for Scientific Papers',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/swsamleo/Paper-Viz',
    author='CRUISE Group',
    author_email='wei.shao@rmit.edu.au',
    license='GNU GPLv3',
    classifiers=[
        'Programming Language :: Python :: 3',
    ],
    keywords=[
        'scientific papers',
        'professional python graphs',
        'thesis template',
        'python',
    ],
	install_requires=[
		'numpy',
		'pandas',
		'matplotlib',
		'seaborn',
	],
    packages=find_packages(),
    include_package_data=True,
)
